/*
  Rhino test framework

   Copyright (C) 2011, 2012  Red Hat

This file is part of IcedTea.

IcedTea is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2, or (at your option)
any later version.

IcedTea is distributed in the hope that it will be useful, but
WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
General Public License for more details.

You should have received a copy of the GNU General Public License
along with IcedTea; see the file COPYING.  If not, write to the
Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
02110-1301 USA.

Linking this library statically or dynamically with other modules is
making a combined work based on this library.  Thus, the terms and
conditions of the GNU General Public License cover the whole
combination.

As a special exception, the copyright holders of this library give you
permission to link this library with independent modules to produce an
executable, regardless of the license terms of these independent
modules, and to copy and distribute the resulting executable under
terms of your choice, provided that you also meet, for each linked
independent module, the terms and conditions of the license of that
module.  An independent module is a module which is not derived from
or based on this library.  If you modify this library, you may extend
this exception to your version of the library, but you are not
obligated to do so.  If you do not wish to do so, delete this
exception statement from your version.
*/

package org.RhinoTests;

import java.util.Arrays;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.HashMap;

import java.lang.reflect.Constructor;
import java.lang.reflect.Field;
import java.lang.reflect.Method;
import java.lang.reflect.Modifier;

import javax.script.AbstractScriptEngine;
import javax.script.ScriptEngineManager;import javax.script.ScriptEngine;



/**
 * Set of tests which check the API of AbstractScriptEngine class using
 * Java reflection API.
 *
 * @author Pavel Tisnovsky
 */
public class AbstractScriptEngineClassTest extends BaseRhinoTest {

    /**
     * Object that represents the type of AbstractScriptEngine.
     */
    Class<?> abstractScriptEngineClass = null;

    @Override
    protected void setUp(String[] args) {
        // setup attribute used by tests
        this.abstractScriptEngineClass = AbstractScriptEngine.class;
    }

    @Override
    protected void tearDown() {
        // this block could be empty
        return;
    }

    /**
     * Test for method javax.script.AbstractScriptEngine.getClass().isAssignableFrom()
     */
    protected void testIsAssignableFrom() {
        assertTrue(this.abstractScriptEngineClass.isAssignableFrom(AbstractScriptEngine.class),
                "Method AbstractScriptEngine.getClass().isAssignableFrom() returns wrong value");
    }

    /**
     * Test for method javax.script.AbstractScriptEngine.getClass().isInstance()
     */
    protected void testIsInstance() {
        assertTrue(this.abstractScriptEngineClass.isInstance(new ScriptEngineManager().getEngineByName(Constants.EngineNames.ENGINE_NAME_JavaScript)),
                "Method AbstractScriptEngine.getClass().isInstance() returns wrong value");
    }

    /**
     * Test for method javax.script.AbstractScriptEngine.getClass().isInterface()
     */
    protected void testIsInterface() {
        assertFalse(this.abstractScriptEngineClass.isInterface(),
                "Method AbstractScriptEngine.getClass().isInterface() returns wrong value");
    }

    /**
     * Test for method javax.script.AbstractScriptEngine.getClass().isLocalClass()
     */
    protected void testIsLocalClass() {
        assertFalse(this.abstractScriptEngineClass.isLocalClass(),
                "Method AbstractScriptEngine.getClass().isLocalClass() returns wrong value");
    }

    /**
     * Test for method javax.script.AbstractScriptEngine.getClass().isMemberClass()
     */
    protected void testIsMemberClass() {
        assertFalse(this.abstractScriptEngineClass.isMemberClass(),
                "Method AbstractScriptEngine.getClass().isMemberClass() returns wrong value");
    }

    /**
     * Test for method javax.script.AbstractScriptEngine.getClass().isPrimitive()
     */
    protected void testIsPrimitive() {
        assertFalse(this.abstractScriptEngineClass.isPrimitive(),
                "Method AbstractScriptEngine.getClass().isPrimitive() returns wrong value");
    }

    /**
     * Test for method javax.script.AbstractScriptEngine.getClass().isSynthetic()
     */
    protected void testIsSynthetic() {
        assertFalse(this.abstractScriptEngineClass.isSynthetic(),
                "Method AbstractScriptEngine.getClass().isSynthetic() returns wrong value");
    }

    /**
     * Test for method javax.script.AbstractScriptEngine.getClass().isAnnotation()
     */
    protected void testIsAnnotation() {
        assertFalse(this.abstractScriptEngineClass.isAnnotation(),
                "Method AbstractScriptEngine.getClass().isAnnotation() returns wrong value");
    }

    /**
     * Test for method javax.script.AbstractScriptEngine.getClass().isAnnotationPresent()
     */
    protected void testIsAnnotationPresent() {
        assertFalse(this.abstractScriptEngineClass.isAnnotationPresent(java.lang.annotation.Annotation.class),
                "Method AbstractScriptEngine.getClass().isAnnotationPresent(java.lang.annotation.Annotation.class) returns wrong value");
        assertFalse(this.abstractScriptEngineClass.isAnnotationPresent(java.lang.annotation.Documented.class),
                "Method AbstractScriptEngine.getClass().isAnnotationPresent(java.lang.annotation.Documented.class) returns wrong value");
        assertFalse(this.abstractScriptEngineClass.isAnnotationPresent(java.lang.annotation.Inherited.class),
                "Method AbstractScriptEngine.getClass().isAnnotationPresent(java.lang.annotation.Inherited.class) returns wrong value");
        assertFalse(this.abstractScriptEngineClass.isAnnotationPresent(java.lang.annotation.Retention.class),
                "Method AbstractScriptEngine.getClass().isAnnotationPresent(java.lang.annotation.Retention.class) returns wrong value");
        assertFalse(this.abstractScriptEngineClass.isAnnotationPresent(java.lang.annotation.Target.class),
                "Method AbstractScriptEngine.getClass().isAnnotationPresent(java.lang.annotation.Target.class) returns wrong value");
        assertFalse(this.abstractScriptEngineClass.isAnnotationPresent(java.lang.Deprecated.class),
                "Method AbstractScriptEngine.getClass().isAnnotationPresent(java.lang.Deprecated.class) returns wrong value");
        assertFalse(this.abstractScriptEngineClass.isAnnotationPresent(java.lang.Override.class),
                "Method AbstractScriptEngine.getClass().isAnnotationPresent(java.lang.Override.class) returns wrong value");
        assertFalse(this.abstractScriptEngineClass.isAnnotationPresent(java.lang.SuppressWarnings.class),
                "Method AbstractScriptEngine.getClass().isAnnotationPresent(java.lang.SuppressWarnings.class) returns wrong value");
    }

    /**
     * Test for method javax.script.AbstractScriptEngine.getClass().isAnonymousClass()
     */
    protected void testIsAnonymousClass() {
        assertFalse(this.abstractScriptEngineClass.isAnonymousClass(),
                "Method AbstractScriptEngine.getClass().isAnonymousClass() returns wrong value");
    }

    /**
     * Test for method javax.script.AbstractScriptEngine.getClass().isArray()
     */
    protected void testIsArray() {
        assertFalse(this.abstractScriptEngineClass.isArray(),
                "Method AbstractScriptEngine.getClass().isArray() returns wrong value");
    }

    /**
     * Test for method javax.script.AbstractScriptEngine.getClass().isEnum()
     */
    protected void testIsEnum() {
        assertFalse(this.abstractScriptEngineClass.isEnum(),
                "Method AbstractScriptEngine.getClass().isEnum() returns wrong value");
    }

    /**
     * Test for method javax.script.AbstractScriptEngine.getClass().getInterfaces()
     */
    protected void testGetInterfaces() {
        List<Class<?>> interfaces = Arrays.asList(this.abstractScriptEngineClass.getInterfaces());
        assertTrue(interfaces.contains(ScriptEngine.class),
                "list of implemented interfaces does not contain ScriptEngine");
    }

    /**
     * Test for method javax.script.AbstractScriptEngine.getClass().getModifiers()
     */
    protected void testGetModifiers() {
        int modifiers = this.abstractScriptEngineClass.getModifiers();
        assertTrue(Modifier.isPublic(modifiers),
                "Method AbstractScriptEngine.getClass().getModifiers() - isPublic modifier is set to a wrong value");
        assertFalse(Modifier.isPrivate(modifiers),
                "Method AbstractScriptEngine.getClass().getModifiers() - isPrivate modifier is set to a wrong value");
        assertFalse(Modifier.isProtected(modifiers),
                "Method AbstractScriptEngine.getClass().getModifiers() - isProtected modifier is set to a wrong value");
        assertTrue(Modifier.isAbstract(modifiers),
                "Method AbstractScriptEngine.getClass().getModifiers() - isAbstract modifier is set to a wrong value");
        assertFalse(Modifier.isFinal(modifiers),
                "Method AbstractScriptEngine.getClass().getModifiers() - isFinal modifier is set to a wrong value");
        assertFalse(Modifier.isInterface(modifiers),
                "Method AbstractScriptEngine.getClass().getModifiers() - isInterface modifier is set to a wrong value");
        assertFalse(Modifier.isNative(modifiers),
                "Method AbstractScriptEngine.getClass().getModifiers() - isNative modifier is set to a wrong value");
        assertFalse(Modifier.isStatic(modifiers),
                "Method AbstractScriptEngine.getClass().getModifiers() - isStatic modifier is set to a wrong value");
        assertFalse(Modifier.isStrict(modifiers),
                "Method AbstractScriptEngine.getClass().getModifiers() - isStrict modifier is set to a wrong value");
        assertFalse(Modifier.isSynchronized(modifiers),
                "Method AbstractScriptEngine.getClass().getModifiers() - isSynchronized modifier is set to a wrong value");
        assertFalse(Modifier.isTransient(modifiers),
                "Method AbstractScriptEngine.getClass().getModifiers() - isTransient modifier is set to a wrong value");
        assertFalse(Modifier.isVolatile(modifiers),
                "Method AbstractScriptEngine.getClass().getModifiers() - isVolatile modifier is set to a wrong value");
    }

    /**
     * Test for method javax.script.AbstractScriptEngine.getClass().getName()
     */
    protected void testGetName() {
        String name = this.abstractScriptEngineClass.getName();
        assertEquals(name, "javax.script.AbstractScriptEngine",
                "Method AbstractScriptEngine.getClass().getName() returns wrong value " + name);
    }

    /**
     * Test for method javax.script.AbstractScriptEngine.getClass().getPackage()
     */
    protected void testGetPackage() {
        Package p = this.abstractScriptEngineClass.getPackage();
        String packageName = p.getName();
        assertEquals(packageName, "javax.script",
                "Method AbstractScriptEngine.getClass().getPackage().getName() returns wrong value " + packageName);
    }

    /**
     * Test for method javax.script.AbstractScriptEngine.getClass().getSimpleName()
     */
    protected void testGetSimpleName() {
        String simpleName = this.abstractScriptEngineClass.getSimpleName();
        assertEquals(simpleName, "AbstractScriptEngine",
                "Method AbstractScriptEngine.getClass().getSimpleName() returns wrong value " + simpleName);
    }

    /**
     * Test for method javax.script.AbstractScriptEngine.getClass().getSuperclass()
     */
    protected void testGetSuperclass() {
        Class<?> superClass = this.abstractScriptEngineClass.getSuperclass();
        String superClassName = superClass.getName();
        assertEquals(superClassName, "java.lang.Object",
                "Method AbstractScriptEngine.getClass().getSuperclass() returns wrong value " + superClassName);
    }

    /**
     * Test for method javax.script.AbstractScriptEngine.getClass().getConstructors()
     */
    protected void testGetConstructors() {
        // map of constructors which should exists
        Map<String, String> testedConstructors = null;
        Map<String, String> testedConstructors_jdk6 = new HashMap<String, String>();
        Map<String, String> testedConstructors_jdk7 = new HashMap<String, String>();

        testedConstructors_jdk6.put("public javax.script.AbstractScriptEngine()", "javax.script.AbstractScriptEngine");
        testedConstructors_jdk6.put("public javax.script.AbstractScriptEngine(javax.script.Bindings)", "javax.script.AbstractScriptEngine");

        testedConstructors_jdk7.put("public javax.script.AbstractScriptEngine()", "javax.script.AbstractScriptEngine");
        testedConstructors_jdk7.put("public javax.script.AbstractScriptEngine(javax.script.Bindings)", "javax.script.AbstractScriptEngine");

        // get the right map containing constructor signatures
        testedConstructors = getJavaVersion() < 7 ? testedConstructors_jdk6 : testedConstructors_jdk7;

        // get all constructors for this class
        Constructor<?>[] constructors = this.abstractScriptEngineClass.getConstructors();

        // basic check for a number of constructors
        assertEquals(constructors.length, 2, "only 2 constructors should be set");

        // check if all constructors exists
        for (Constructor<?> constructor : constructors) {
            String constructorName = constructor.getName();
            String constructorString = constructor.toString();
            assertTrue(testedConstructors.containsKey(constructorString), "wrong constructor.toString() " + constructorName);
            assertEquals(testedConstructors.get(constructorString), constructorName, "wrong constructor name " + constructorName);
        }
    }

    /**
     * Test for method javax.script.AbstractScriptEngine.getClass().getDeclaredConstructors()
     */
    protected void testGetDeclaredConstructors() {
        // map of constructors which should exists
        Map<String, String> testedConstructors = null;
        Map<String, String> testedConstructors_jdk6 = new HashMap<String, String>();
        Map<String, String> testedConstructors_jdk7 = new HashMap<String, String>();

        testedConstructors_jdk6.put("public javax.script.AbstractScriptEngine()", "javax.script.AbstractScriptEngine");
        testedConstructors_jdk6.put("public javax.script.AbstractScriptEngine(javax.script.Bindings)", "javax.script.AbstractScriptEngine");

        testedConstructors_jdk7.put("public javax.script.AbstractScriptEngine()", "javax.script.AbstractScriptEngine");
        testedConstructors_jdk7.put("public javax.script.AbstractScriptEngine(javax.script.Bindings)", "javax.script.AbstractScriptEngine");

        // get the right map containing constructor signatures
        testedConstructors = getJavaVersion() < 7 ? testedConstructors_jdk6 : testedConstructors_jdk7;

        // get all declared constructors for this class
        Constructor<?>[] declaredConstructors = this.abstractScriptEngineClass.getDeclaredConstructors();

        // basic check for a number of declared constructors
        assertEquals(declaredConstructors.length, 2, "only 2 constructors should be set");

        // check if all declared constructors exists
        for (Constructor<?> declaredConstructor : declaredConstructors) {
            String constructorName = declaredConstructor.getName();
            String constructorString = declaredConstructor.toString();
            assertTrue(testedConstructors.containsKey(constructorString), "wrong constructor.toString() " + constructorName);
            assertEquals(testedConstructors.get(constructorString), constructorName, "wrong constructor name " + constructorName);
        }
    }

    /**
     * Test for method javax.script.AbstractScriptEngine.getClass().getFields()
     */
    protected void testGetFields() {
        // following fields should exists
        final String[] fieldsThatShouldExists = {
            "public static final java.lang.String javax.script.ScriptEngine.ARGV",
            "public static final java.lang.String javax.script.ScriptEngine.FILENAME",
            "public static final java.lang.String javax.script.ScriptEngine.ENGINE",
            "public static final java.lang.String javax.script.ScriptEngine.ENGINE_VERSION",
            "public static final java.lang.String javax.script.ScriptEngine.NAME",
            "public static final java.lang.String javax.script.ScriptEngine.LANGUAGE",
            "public static final java.lang.String javax.script.ScriptEngine.LANGUAGE_VERSION",
        };
        // get all fields
        Field[] fields = this.abstractScriptEngineClass.getFields();
        // and transform the array into a list of field names
        List<String> fieldsAsString = new ArrayList<String>();
        for (Field field : fields) {
            fieldsAsString.add(field.toString());
        }
        // check if all required fields really exists
        for (String fieldThatShouldExists : fieldsThatShouldExists) {
            assertTrue(fieldsAsString.contains(fieldThatShouldExists),
                    "field " + fieldThatShouldExists + " not found");
        }
    }

    /**
     * Test for method javax.script.AbstractScriptEngine.getClass().getDeclaredFields()
     */
    protected void testGetDeclaredFields() {
        // following fields should be declared
        final String[] fieldsThatShouldExists = {
            "protected javax.script.ScriptContext javax.script.AbstractScriptEngine.context",
        };
        // get all declared fields
        Field[] declaredFields = this.abstractScriptEngineClass.getDeclaredFields();
        // and transform the array into a list of field names
        List<String> declaredFieldsAsString = new ArrayList<String>();
        for (Field field : declaredFields) {
            declaredFieldsAsString.add(field.toString());
        }
        // check if all required fields really exists
        for (String fieldThatShouldExists : fieldsThatShouldExists) {
            assertTrue(declaredFieldsAsString.contains(fieldThatShouldExists),
                    "field " + fieldThatShouldExists + " not found");
        }
    }

    /**
     * Test for method javax.script.AbstractScriptEngine.getClass().getMethods()
     */
    protected void testGetMethods() {
        // following methods should be inherited
        final String[] methodsThatShouldExists_jdk6 = {
            "public abstract java.lang.Object javax.script.ScriptEngine.eval(java.io.Reader,javax.script.ScriptContext) throws javax.script.ScriptException",
            "public abstract java.lang.Object javax.script.ScriptEngine.eval(java.lang.String,javax.script.ScriptContext) throws javax.script.ScriptException",
            "public abstract javax.script.Bindings javax.script.ScriptEngine.createBindings()",
            "public abstract javax.script.ScriptEngineFactory javax.script.ScriptEngine.getFactory()",
            "public boolean java.lang.Object.equals(java.lang.Object)",
            "public final native java.lang.Class java.lang.Object.getClass()",
            "public final native void java.lang.Object.notify()",
            "public final native void java.lang.Object.notifyAll()",
            "public final native void java.lang.Object.wait(long) throws java.lang.InterruptedException",
            "public final void java.lang.Object.wait() throws java.lang.InterruptedException",
            "public final void java.lang.Object.wait(long,int) throws java.lang.InterruptedException",
            "public java.lang.Object javax.script.AbstractScriptEngine.eval(java.io.Reader) throws javax.script.ScriptException",
            "public java.lang.Object javax.script.AbstractScriptEngine.eval(java.io.Reader,javax.script.Bindings) throws javax.script.ScriptException",
            "public java.lang.Object javax.script.AbstractScriptEngine.eval(java.lang.String) throws javax.script.ScriptException",
            "public java.lang.Object javax.script.AbstractScriptEngine.eval(java.lang.String,javax.script.Bindings) throws javax.script.ScriptException",
            "public java.lang.Object javax.script.AbstractScriptEngine.get(java.lang.String)",
            "public java.lang.String java.lang.Object.toString()",
            "public javax.script.Bindings javax.script.AbstractScriptEngine.getBindings(int)",
            "public javax.script.ScriptContext javax.script.AbstractScriptEngine.getContext()",
            "public native int java.lang.Object.hashCode()",
            "public void javax.script.AbstractScriptEngine.put(java.lang.String,java.lang.Object)",
            "public void javax.script.AbstractScriptEngine.setBindings(javax.script.Bindings,int)",
            "public void javax.script.AbstractScriptEngine.setContext(javax.script.ScriptContext)",
        };

        final String[] methodsThatShouldExists_jdk7 = {
            "public abstract java.lang.Object javax.script.ScriptEngine.eval(java.io.Reader,javax.script.ScriptContext) throws javax.script.ScriptException",
            "public abstract java.lang.Object javax.script.ScriptEngine.eval(java.lang.String,javax.script.ScriptContext) throws javax.script.ScriptException",
            "public abstract javax.script.Bindings javax.script.ScriptEngine.createBindings()",
            "public abstract javax.script.ScriptEngineFactory javax.script.ScriptEngine.getFactory()",
            "public boolean java.lang.Object.equals(java.lang.Object)",
            "public final native java.lang.Class java.lang.Object.getClass()",
            "public final native void java.lang.Object.notify()",
            "public final native void java.lang.Object.notifyAll()",
            "public final native void java.lang.Object.wait(long) throws java.lang.InterruptedException",
            "public final void java.lang.Object.wait() throws java.lang.InterruptedException",
            "public final void java.lang.Object.wait(long,int) throws java.lang.InterruptedException",
            "public java.lang.Object javax.script.AbstractScriptEngine.eval(java.io.Reader) throws javax.script.ScriptException",
            "public java.lang.Object javax.script.AbstractScriptEngine.eval(java.io.Reader,javax.script.Bindings) throws javax.script.ScriptException",
            "public java.lang.Object javax.script.AbstractScriptEngine.eval(java.lang.String) throws javax.script.ScriptException",
            "public java.lang.Object javax.script.AbstractScriptEngine.eval(java.lang.String,javax.script.Bindings) throws javax.script.ScriptException",
            "public java.lang.Object javax.script.AbstractScriptEngine.get(java.lang.String)",
            "public java.lang.String java.lang.Object.toString()",
            "public javax.script.Bindings javax.script.AbstractScriptEngine.getBindings(int)",
            "public javax.script.ScriptContext javax.script.AbstractScriptEngine.getContext()",
            "public native int java.lang.Object.hashCode()",
            "public void javax.script.AbstractScriptEngine.put(java.lang.String,java.lang.Object)",
            "public void javax.script.AbstractScriptEngine.setBindings(javax.script.Bindings,int)",
            "public void javax.script.AbstractScriptEngine.setContext(javax.script.ScriptContext)",
        };

        // get all inherited methods
        Method[] methods = this.abstractScriptEngineClass.getMethods();
        // and transform the array into a list of method names
        List<String> methodsAsString = new ArrayList<String>();
        for (Method method : methods) {
            methodsAsString.add(method.toString());
        }
        String[] methodsThatShouldExists = getJavaVersion() < 7 ? methodsThatShouldExists_jdk6 : methodsThatShouldExists_jdk7;
        // check if all required methods really exists
        for (String methodThatShouldExists : methodsThatShouldExists) {
            assertTrue(methodsAsString.contains(methodThatShouldExists),
                    "method " + methodThatShouldExists + " not found");
        }
    }

    /**
     * Test for method javax.script.AbstractScriptEngine.getClass().getDeclaredMethods()
     */
    protected void testGetDeclaredMethods() {
        // following methods should be declared
        final String[] declaredMethodsThatShouldExists_jdk6 = {
            "protected javax.script.ScriptContext javax.script.AbstractScriptEngine.getScriptContext(javax.script.Bindings)",
            "public java.lang.Object javax.script.AbstractScriptEngine.eval(java.io.Reader) throws javax.script.ScriptException",
            "public java.lang.Object javax.script.AbstractScriptEngine.eval(java.io.Reader,javax.script.Bindings) throws javax.script.ScriptException",
            "public java.lang.Object javax.script.AbstractScriptEngine.eval(java.lang.String) throws javax.script.ScriptException",
            "public java.lang.Object javax.script.AbstractScriptEngine.eval(java.lang.String,javax.script.Bindings) throws javax.script.ScriptException",
            "public java.lang.Object javax.script.AbstractScriptEngine.get(java.lang.String)",
            "public javax.script.Bindings javax.script.AbstractScriptEngine.getBindings(int)",
            "public javax.script.ScriptContext javax.script.AbstractScriptEngine.getContext()",
            "public void javax.script.AbstractScriptEngine.put(java.lang.String,java.lang.Object)",
            "public void javax.script.AbstractScriptEngine.setBindings(javax.script.Bindings,int)",
            "public void javax.script.AbstractScriptEngine.setContext(javax.script.ScriptContext)",
        };

        final String[] declaredMethodsThatShouldExists_jdk7 = {
            "protected javax.script.ScriptContext javax.script.AbstractScriptEngine.getScriptContext(javax.script.Bindings)",
            "public java.lang.Object javax.script.AbstractScriptEngine.eval(java.io.Reader) throws javax.script.ScriptException",
            "public java.lang.Object javax.script.AbstractScriptEngine.eval(java.io.Reader,javax.script.Bindings) throws javax.script.ScriptException",
            "public java.lang.Object javax.script.AbstractScriptEngine.eval(java.lang.String) throws javax.script.ScriptException",
            "public java.lang.Object javax.script.AbstractScriptEngine.eval(java.lang.String,javax.script.Bindings) throws javax.script.ScriptException",
            "public java.lang.Object javax.script.AbstractScriptEngine.get(java.lang.String)",
            "public javax.script.Bindings javax.script.AbstractScriptEngine.getBindings(int)",
            "public javax.script.ScriptContext javax.script.AbstractScriptEngine.getContext()",
            "public void javax.script.AbstractScriptEngine.put(java.lang.String,java.lang.Object)",
            "public void javax.script.AbstractScriptEngine.setBindings(javax.script.Bindings,int)",
            "public void javax.script.AbstractScriptEngine.setContext(javax.script.ScriptContext)",
        };

        // get all declared methods
        Method[] declaredMethods = this.abstractScriptEngineClass.getDeclaredMethods();
        // and transform the array into a list of method names
        List<String> methodsAsString = new ArrayList<String>();
        for (Method method : declaredMethods) {
            methodsAsString.add(method.toString());
        }
        String[] declaredMethodsThatShouldExists = getJavaVersion() < 7 ? declaredMethodsThatShouldExists_jdk6 : declaredMethodsThatShouldExists_jdk7;
        // check if all required methods really exists
        for (String methodThatShouldExists : declaredMethodsThatShouldExists) {
            assertTrue(methodsAsString.contains(methodThatShouldExists),
                    "declared method " + methodThatShouldExists + " not found");
        }
    }

    /**
     * Test for instanceof operator applied to a class javax.script.AbstractScriptEngine
     */
    @SuppressWarnings("cast")
    protected void testInstanceOf() {
        // tested object
        Object o = new ScriptEngineManager().getEngineByName(Constants.EngineNames.ENGINE_NAME_JavaScript);

        // basic check of instanceof operator
        assertTrue(o instanceof AbstractScriptEngine, "instanceof AbstractScriptEngine is wrongly evaluated to false");

        // check operator instanceof against all superclasses
        assertTrue(o instanceof Object, "instanceof Object is wrongly evaluated to false");
    }

    /**
     * Entry point to this test case.
     *
     * @param args parameters passed from command line
     */
    public static void main(String[] args) {
        new AbstractScriptEngineClassTest().doTests(args);
    }
}

