/*
  Rhino test framework

   Copyright (C) 2011, 2012  Red Hat

This file is part of IcedTea.

IcedTea is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2, or (at your option)
any later version.

IcedTea is distributed in the hope that it will be useful, but
WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
General Public License for more details.

You should have received a copy of the GNU General Public License
along with IcedTea; see the file COPYING.  If not, write to the
Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
02110-1301 USA.

Linking this library statically or dynamically with other modules is
making a combined work based on this library.  Thus, the terms and
conditions of the GNU General Public License cover the whole
combination.

As a special exception, the copyright holders of this library give you
permission to link this library with independent modules to produce an
executable, regardless of the license terms of these independent
modules, and to copy and distribute the resulting executable under
terms of your choice, provided that you also meet, for each linked
independent module, the terms and conditions of the license of that
module.  An independent module is a module which is not derived from
or based on this library.  If you modify this library, you may extend
this exception to your version of the library, but you are not
obligated to do so.  If you do not wish to do so, delete this
exception statement from your version.
 */

package org.RhinoTests.Reporter;

import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

/**
 * 
 * @author Pavel Tisnovsky
 */
public class IndexPageGenerator
{

	public static void generate(Map<String, Map<String, List<String>>> testResults, CommandLineParameters params) {
		String date = params.getDate();
        if (!testResults.containsKey(params.getDate())) {
            throw new RuntimeException("Results for " + date + " don't exist");
        }
        Map<String, List<String>> testResultForDate = testResults.get(date);
        TestResult testResult = TestResult.readSummary(testResultForDate);
        testResult.setDate(date);
        createIndexPageFromTemplate(testResult, params);
    }

    private static void createIndexPageFromTemplate(TestResult testResult, CommandLineParameters params) {
        List<String> template = FileUtils.readTextFile(params.getTemplateDir() + "/index.html");
        List<String> out = new LinkedList<String>();
        // iterate through whole template
        Pattern pattern = Pattern.compile("\\$\\{.*\\}");
        for (String templateLine : template) {
        	out.add(replacePlaceholder(testResult, pattern, templateLine));
        }
        // write list of string to a file with given name
        FileUtils.writeTextFile(params.getReportDir() + "/index.html", out);
    }

	private static String replacePlaceholder(TestResult testResult, Pattern pattern, String templateLine) {
		Matcher matcher = pattern.matcher(templateLine);
		// if some pattern ${PLACEHOLDER} is found on the template line,
		// it is replaced by the result value of method TestResult.getPlaceholder()
		if (matcher.find()) {
			String matcherGroup = matcher.group();
			String placeholderName = StringUtils.getPlaceholderMethodName(matcherGroup);
			try {
				Method method = testResult.getClass().getMethod(placeholderName);
				Object result = method.invoke(testResult);
				if (result != null) {
					return templateLine.replace(matcherGroup, result.toString());
				}
			} catch (SecurityException e) {
				e.printStackTrace();
			} catch (NoSuchMethodException e) {
				e.printStackTrace();
			} catch (IllegalArgumentException e) {
				e.printStackTrace();
			} catch (IllegalAccessException e) {
				e.printStackTrace();
			} catch (InvocationTargetException e) {
				e.printStackTrace();
			}
		}
		return templateLine;
	}

}
