/*
  Rhino test framework

   Copyright (C) 2011, 2012  Red Hat

This file is part of IcedTea.

IcedTea is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2, or (at your option)
any later version.

IcedTea is distributed in the hope that it will be useful, but
WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
General Public License for more details.

You should have received a copy of the GNU General Public License
along with IcedTea; see the file COPYING.  If not, write to the
Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
02110-1301 USA.

Linking this library statically or dynamically with other modules is
making a combined work based on this library.  Thus, the terms and
conditions of the GNU General Public License cover the whole
combination.

As a special exception, the copyright holders of this library give you
permission to link this library with independent modules to produce an
executable, regardless of the license terms of these independent
modules, and to copy and distribute the resulting executable under
terms of your choice, provided that you also meet, for each linked
independent module, the terms and conditions of the license of that
module.  An independent module is a module which is not derived from
or based on this library.  If you modify this library, you may extend
this exception to your version of the library, but you are not
obligated to do so.  If you do not wish to do so, delete this
exception statement from your version.
*/

package org.RhinoTests;

import java.util.Arrays;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.HashMap;

import java.lang.reflect.Constructor;
import java.lang.reflect.Field;
import java.lang.reflect.Method;
import java.lang.reflect.Modifier;

import javax.script.ScriptContext;
import javax.script.SimpleScriptContext;



/**
 * Set of tests which check the API of ScriptContext interface using
 * Java reflection API.
 *
 * @author Pavel Tisnovsky
 */
public class ScriptContextClassTest extends BaseRhinoTest {

    /**
     * Object that represents the type of ScriptContext.
     */
    Class<?> scriptContextClass = null;

    @Override
    protected void setUp(String[] args) {
        // setup attribute used by tests
        this.scriptContextClass = ScriptContext.class;
    }

    @Override
    protected void tearDown() {
        // this block could be empty
        return;
    }

    /**
     * Test for method javax.script.ScriptContext.getClass().isAssignableFrom()
     */
    protected void testIsAssignableFrom() {
        assertTrue(this.scriptContextClass.isAssignableFrom(ScriptContext.class),
                "Method ScriptContext.getClass().isAssignableFrom() returns wrong value");
    }

    /**
     * Test for method javax.script.ScriptContext.getClass().isInstance()
     */
    protected void testIsInstance() {
        assertTrue(this.scriptContextClass.isInstance(new SimpleScriptContext()),
                "Method ScriptContext.getClass().isInstance() returns wrong value");
    }

    /**
     * Test for method javax.script.ScriptContext.getClass().isInterface()
     */
    protected void testIsInterface() {
        assertTrue(this.scriptContextClass.isInterface(),
                "Method ScriptContext.getClass().isInterface() returns wrong value");
    }

    /**
     * Test for method javax.script.ScriptContext.getClass().isLocalClass()
     */
    protected void testIsLocalClass() {
        assertFalse(this.scriptContextClass.isLocalClass(),
                "Method ScriptContext.getClass().isLocalClass() returns wrong value");
    }

    /**
     * Test for method javax.script.ScriptContext.getClass().isMemberClass()
     */
    protected void testIsMemberClass() {
        assertFalse(this.scriptContextClass.isMemberClass(),
                "Method ScriptContext.getClass().isMemberClass() returns wrong value");
    }

    /**
     * Test for method javax.script.ScriptContext.getClass().isPrimitive()
     */
    protected void testIsPrimitive() {
        assertFalse(this.scriptContextClass.isPrimitive(),
                "Method ScriptContext.getClass().isPrimitive() returns wrong value");
    }

    /**
     * Test for method javax.script.ScriptContext.getClass().isSynthetic()
     */
    protected void testIsSynthetic() {
        assertFalse(this.scriptContextClass.isSynthetic(),
                "Method ScriptContext.getClass().isSynthetic() returns wrong value");
    }

    /**
     * Test for method javax.script.ScriptContext.getClass().isAnnotation()
     */
    protected void testIsAnnotation() {
        assertFalse(this.scriptContextClass.isAnnotation(),
                "Method ScriptContext.getClass().isAnnotation() returns wrong value");
    }

    /**
     * Test for method javax.script.ScriptContext.getClass().isAnnotationPresent()
     */
    protected void testIsAnnotationPresent() {
        assertFalse(this.scriptContextClass.isAnnotationPresent(java.lang.annotation.Annotation.class),
                "Method ScriptContext.getClass().isAnnotationPresent(java.lang.annotation.Annotation.class) returns wrong value");
        assertFalse(this.scriptContextClass.isAnnotationPresent(java.lang.annotation.Documented.class),
                "Method ScriptContext.getClass().isAnnotationPresent(java.lang.annotation.Documented.class) returns wrong value");
        assertFalse(this.scriptContextClass.isAnnotationPresent(java.lang.annotation.Inherited.class),
                "Method ScriptContext.getClass().isAnnotationPresent(java.lang.annotation.Inherited.class) returns wrong value");
        assertFalse(this.scriptContextClass.isAnnotationPresent(java.lang.annotation.Retention.class),
                "Method ScriptContext.getClass().isAnnotationPresent(java.lang.annotation.Retention.class) returns wrong value");
        assertFalse(this.scriptContextClass.isAnnotationPresent(java.lang.annotation.Target.class),
                "Method ScriptContext.getClass().isAnnotationPresent(java.lang.annotation.Target.class) returns wrong value");
        assertFalse(this.scriptContextClass.isAnnotationPresent(java.lang.Deprecated.class),
                "Method ScriptContext.getClass().isAnnotationPresent(java.lang.Deprecated.class) returns wrong value");
        assertFalse(this.scriptContextClass.isAnnotationPresent(java.lang.Override.class),
                "Method ScriptContext.getClass().isAnnotationPresent(java.lang.Override.class) returns wrong value");
        assertFalse(this.scriptContextClass.isAnnotationPresent(java.lang.SuppressWarnings.class),
                "Method ScriptContext.getClass().isAnnotationPresent(java.lang.SuppressWarnings.class) returns wrong value");
    }

    /**
     * Test for method javax.script.ScriptContext.getClass().isAnonymousClass()
     */
    protected void testIsAnonymousClass() {
        assertFalse(this.scriptContextClass.isAnonymousClass(),
                "Method ScriptContext.getClass().isAnonymousClass() returns wrong value");
    }

    /**
     * Test for method javax.script.ScriptContext.getClass().isArray()
     */
    protected void testIsArray() {
        assertFalse(this.scriptContextClass.isArray(),
                "Method ScriptContext.getClass().isArray() returns wrong value");
    }

    /**
     * Test for method javax.script.ScriptContext.getClass().isEnum()
     */
    protected void testIsEnum() {
        assertFalse(this.scriptContextClass.isEnum(),
                "Method ScriptContext.getClass().isEnum() returns wrong value");
    }

    /**
     * Test for method javax.script.ScriptContext.getClass().getInterfaces()
     */
    protected void testGetInterfaces() {
        List<Class<?>> interfaces = Arrays.asList(this.scriptContextClass.getInterfaces());
        assertTrue(interfaces.isEmpty(),
                "list of implemented interfaces should be empty");
    }

    /**
     * Test for method javax.script.ScriptContext.getClass().getModifiers()
     */
    protected void testGetModifiers() {
        int modifiers = this.scriptContextClass.getModifiers();
        assertTrue(Modifier.isPublic(modifiers),
                "Method ScriptContext.getClass().getModifiers() - isPublic modifier is set to a wrong value");
        assertFalse(Modifier.isPrivate(modifiers),
                "Method ScriptContext.getClass().getModifiers() - isPrivate modifier is set to a wrong value");
        assertFalse(Modifier.isProtected(modifiers),
                "Method ScriptContext.getClass().getModifiers() - isProtected modifier is set to a wrong value");
        assertTrue(Modifier.isAbstract(modifiers),
                "Method ScriptContext.getClass().getModifiers() - isAbstract modifier is set to a wrong value");
        assertFalse(Modifier.isFinal(modifiers),
                "Method ScriptContext.getClass().getModifiers() - isFinal modifier is set to a wrong value");
        assertTrue(Modifier.isInterface(modifiers),
                "Method ScriptContext.getClass().getModifiers() - isInterface modifier is set to a wrong value");
        assertFalse(Modifier.isNative(modifiers),
                "Method ScriptContext.getClass().getModifiers() - isNative modifier is set to a wrong value");
        assertFalse(Modifier.isStatic(modifiers),
                "Method ScriptContext.getClass().getModifiers() - isStatic modifier is set to a wrong value");
        assertFalse(Modifier.isStrict(modifiers),
                "Method ScriptContext.getClass().getModifiers() - isStrict modifier is set to a wrong value");
        assertFalse(Modifier.isSynchronized(modifiers),
                "Method ScriptContext.getClass().getModifiers() - isSynchronized modifier is set to a wrong value");
        assertFalse(Modifier.isTransient(modifiers),
                "Method ScriptContext.getClass().getModifiers() - isTransient modifier is set to a wrong value");
        assertFalse(Modifier.isVolatile(modifiers),
                "Method ScriptContext.getClass().getModifiers() - isVolatile modifier is set to a wrong value");
    }

    /**
     * Test for method javax.script.ScriptContext.getClass().getName()
     */
    protected void testGetName() {
        String name = this.scriptContextClass.getName();
        assertEquals(name, "javax.script.ScriptContext",
                "Method ScriptContext.getClass().getName() returns wrong value " + name);
    }

    /**
     * Test for method javax.script.ScriptContext.getClass().getPackage()
     */
    protected void testGetPackage() {
        Package p = this.scriptContextClass.getPackage();
        String packageName = p.getName();
        assertEquals(packageName, "javax.script",
                "Method ScriptContext.getClass().getPackage().getName() returns wrong value " + packageName);
    }

    /**
     * Test for method javax.script.ScriptContext.getClass().getSimpleName()
     */
    protected void testGetSimpleName() {
        String simpleName = this.scriptContextClass.getSimpleName();
        assertEquals(simpleName, "ScriptContext",
                "Method ScriptContext.getClass().getSimpleName() returns wrong value " + simpleName);
    }

    /**
     * Test for method javax.script.ScriptContext.getClass().getSuperclass()
     */
    protected void testGetSuperclass() {
        Class<?> superClass = this.scriptContextClass.getSuperclass();
        assertNull(superClass,
                "Method ScriptContext.getClass().getSuperclass() does not return null");
    }

    /**
     * Test for method javax.script.ScriptContext.getClass().getConstructors()
     */
    protected void testGetConstructors() {
        // map of constructors which should exists
        Map<String, String> testedConstructors = null;
        Map<String, String> testedConstructors_jdk6 = new HashMap<String, String>();
        Map<String, String> testedConstructors_jdk7 = new HashMap<String, String>();


        // get the right map containing constructor signatures
        testedConstructors = getJavaVersion() < 7 ? testedConstructors_jdk6 : testedConstructors_jdk7;

        // get all constructors for this class
        Constructor<?>[] constructors = this.scriptContextClass.getConstructors();

        // basic check for a number of constructors
        assertEquals(constructors.length, 0, "no constructors should be set");

    }

    /**
     * Test for method javax.script.ScriptContext.getClass().getDeclaredConstructors()
     */
    protected void testGetDeclaredConstructors() {
        // map of constructors which should exists
        Map<String, String> testedConstructors = null;
        Map<String, String> testedConstructors_jdk6 = new HashMap<String, String>();
        Map<String, String> testedConstructors_jdk7 = new HashMap<String, String>();


        // get the right map containing constructor signatures
        testedConstructors = getJavaVersion() < 7 ? testedConstructors_jdk6 : testedConstructors_jdk7;

        // get all declared constructors for this class
        Constructor<?>[] declaredConstructors = this.scriptContextClass.getDeclaredConstructors();

        // basic check for a number of declared constructors
        assertEquals(declaredConstructors.length, 0, "no constructors should be set");

    }

    /**
     * Test for method javax.script.ScriptContext.getClass().getFields()
     */
    protected void testGetFields() {
        // following fields should exists
        final String[] fieldsThatShouldExists = {
            "public static final int javax.script.ScriptContext.ENGINE_SCOPE",
            "public static final int javax.script.ScriptContext.GLOBAL_SCOPE",
        };
        // get all fields
        Field[] fields = this.scriptContextClass.getFields();
        // and transform the array into a list of field names
        List<String> fieldsAsString = new ArrayList<String>();
        for (Field field : fields) {
            fieldsAsString.add(field.toString());
        }
        // check if all required fields really exists
        for (String fieldThatShouldExists : fieldsThatShouldExists) {
            assertTrue(fieldsAsString.contains(fieldThatShouldExists),
                    "field " + fieldThatShouldExists + " not found");
        }
    }

    /**
     * Test for method javax.script.ScriptContext.getClass().getDeclaredFields()
     */
    protected void testGetDeclaredFields() {
        // following fields should be declared
        final String[] fieldsThatShouldExists = {
            "public static final int javax.script.ScriptContext.ENGINE_SCOPE",
            "public static final int javax.script.ScriptContext.GLOBAL_SCOPE",
        };
        // get all declared fields
        Field[] declaredFields = this.scriptContextClass.getDeclaredFields();
        // and transform the array into a list of field names
        List<String> declaredFieldsAsString = new ArrayList<String>();
        for (Field field : declaredFields) {
            declaredFieldsAsString.add(field.toString());
        }
        // check if all required fields really exists
        for (String fieldThatShouldExists : fieldsThatShouldExists) {
            assertTrue(declaredFieldsAsString.contains(fieldThatShouldExists),
                    "field " + fieldThatShouldExists + " not found");
        }
    }

    /**
     * Test for method javax.script.ScriptContext.getClass().getMethods()
     */
    protected void testGetMethods() {
        // following methods should be inherited
        final String[] methodsThatShouldExists_jdk6 = {
            "public abstract int javax.script.ScriptContext.getAttributesScope(java.lang.String)",
            "public abstract java.io.Reader javax.script.ScriptContext.getReader()",
            "public abstract java.io.Writer javax.script.ScriptContext.getErrorWriter()",
            "public abstract java.io.Writer javax.script.ScriptContext.getWriter()",
            "public abstract java.lang.Object javax.script.ScriptContext.getAttribute(java.lang.String)",
            "public abstract java.lang.Object javax.script.ScriptContext.getAttribute(java.lang.String,int)",
            "public abstract java.lang.Object javax.script.ScriptContext.removeAttribute(java.lang.String,int)",
            "public abstract java.util.List javax.script.ScriptContext.getScopes()",
            "public abstract javax.script.Bindings javax.script.ScriptContext.getBindings(int)",
            "public abstract void javax.script.ScriptContext.setAttribute(java.lang.String,java.lang.Object,int)",
            "public abstract void javax.script.ScriptContext.setBindings(javax.script.Bindings,int)",
            "public abstract void javax.script.ScriptContext.setErrorWriter(java.io.Writer)",
            "public abstract void javax.script.ScriptContext.setReader(java.io.Reader)",
            "public abstract void javax.script.ScriptContext.setWriter(java.io.Writer)",
        };

        final String[] methodsThatShouldExists_jdk7 = {
            "public abstract int javax.script.ScriptContext.getAttributesScope(java.lang.String)",
            "public abstract java.io.Reader javax.script.ScriptContext.getReader()",
            "public abstract java.io.Writer javax.script.ScriptContext.getErrorWriter()",
            "public abstract java.io.Writer javax.script.ScriptContext.getWriter()",
            "public abstract java.lang.Object javax.script.ScriptContext.getAttribute(java.lang.String)",
            "public abstract java.lang.Object javax.script.ScriptContext.getAttribute(java.lang.String,int)",
            "public abstract java.lang.Object javax.script.ScriptContext.removeAttribute(java.lang.String,int)",
            "public abstract java.util.List javax.script.ScriptContext.getScopes()",
            "public abstract javax.script.Bindings javax.script.ScriptContext.getBindings(int)",
            "public abstract void javax.script.ScriptContext.setAttribute(java.lang.String,java.lang.Object,int)",
            "public abstract void javax.script.ScriptContext.setBindings(javax.script.Bindings,int)",
            "public abstract void javax.script.ScriptContext.setErrorWriter(java.io.Writer)",
            "public abstract void javax.script.ScriptContext.setReader(java.io.Reader)",
            "public abstract void javax.script.ScriptContext.setWriter(java.io.Writer)",
        };

        // get all inherited methods
        Method[] methods = this.scriptContextClass.getMethods();
        // and transform the array into a list of method names
        List<String> methodsAsString = new ArrayList<String>();
        for (Method method : methods) {
            methodsAsString.add(method.toString());
        }
        String[] methodsThatShouldExists = getJavaVersion() < 7 ? methodsThatShouldExists_jdk6 : methodsThatShouldExists_jdk7;
        // check if all required methods really exists
        for (String methodThatShouldExists : methodsThatShouldExists) {
            assertTrue(methodsAsString.contains(methodThatShouldExists),
                    "method " + methodThatShouldExists + " not found");
        }
    }

    /**
     * Test for method javax.script.ScriptContext.getClass().getDeclaredMethods()
     */
    protected void testGetDeclaredMethods() {
        // following methods should be declared
        final String[] declaredMethodsThatShouldExists_jdk6 = {
            "public abstract int javax.script.ScriptContext.getAttributesScope(java.lang.String)",
            "public abstract java.io.Reader javax.script.ScriptContext.getReader()",
            "public abstract java.io.Writer javax.script.ScriptContext.getErrorWriter()",
            "public abstract java.io.Writer javax.script.ScriptContext.getWriter()",
            "public abstract java.lang.Object javax.script.ScriptContext.getAttribute(java.lang.String)",
            "public abstract java.lang.Object javax.script.ScriptContext.getAttribute(java.lang.String,int)",
            "public abstract java.lang.Object javax.script.ScriptContext.removeAttribute(java.lang.String,int)",
            "public abstract java.util.List javax.script.ScriptContext.getScopes()",
            "public abstract javax.script.Bindings javax.script.ScriptContext.getBindings(int)",
            "public abstract void javax.script.ScriptContext.setAttribute(java.lang.String,java.lang.Object,int)",
            "public abstract void javax.script.ScriptContext.setBindings(javax.script.Bindings,int)",
            "public abstract void javax.script.ScriptContext.setErrorWriter(java.io.Writer)",
            "public abstract void javax.script.ScriptContext.setReader(java.io.Reader)",
            "public abstract void javax.script.ScriptContext.setWriter(java.io.Writer)",
        };

        final String[] declaredMethodsThatShouldExists_jdk7 = {
            "public abstract int javax.script.ScriptContext.getAttributesScope(java.lang.String)",
            "public abstract java.io.Reader javax.script.ScriptContext.getReader()",
            "public abstract java.io.Writer javax.script.ScriptContext.getErrorWriter()",
            "public abstract java.io.Writer javax.script.ScriptContext.getWriter()",
            "public abstract java.lang.Object javax.script.ScriptContext.getAttribute(java.lang.String)",
            "public abstract java.lang.Object javax.script.ScriptContext.getAttribute(java.lang.String,int)",
            "public abstract java.lang.Object javax.script.ScriptContext.removeAttribute(java.lang.String,int)",
            "public abstract java.util.List javax.script.ScriptContext.getScopes()",
            "public abstract javax.script.Bindings javax.script.ScriptContext.getBindings(int)",
            "public abstract void javax.script.ScriptContext.setAttribute(java.lang.String,java.lang.Object,int)",
            "public abstract void javax.script.ScriptContext.setBindings(javax.script.Bindings,int)",
            "public abstract void javax.script.ScriptContext.setErrorWriter(java.io.Writer)",
            "public abstract void javax.script.ScriptContext.setReader(java.io.Reader)",
            "public abstract void javax.script.ScriptContext.setWriter(java.io.Writer)",
        };

        // get all declared methods
        Method[] declaredMethods = this.scriptContextClass.getDeclaredMethods();
        // and transform the array into a list of method names
        List<String> methodsAsString = new ArrayList<String>();
        for (Method method : declaredMethods) {
            methodsAsString.add(method.toString());
        }
        String[] declaredMethodsThatShouldExists = getJavaVersion() < 7 ? declaredMethodsThatShouldExists_jdk6 : declaredMethodsThatShouldExists_jdk7;
        // check if all required methods really exists
        for (String methodThatShouldExists : declaredMethodsThatShouldExists) {
            assertTrue(methodsAsString.contains(methodThatShouldExists),
                    "declared method " + methodThatShouldExists + " not found");
        }
    }

    /**
     * Test for instanceof operator applied to a class javax.script.ScriptContext
     */
    @SuppressWarnings("cast")
    protected void testInstanceOf() {
        // tested object
        Object o = new SimpleScriptContext();

        // basic check of instanceof operator
        assertTrue(o instanceof ScriptContext, "instanceof ScriptContext is wrongly evaluated to false");

        // check operator instanceof against all superclasses
        assertTrue(o instanceof Object, "instanceof Object is wrongly evaluated to false");
    }

    /**
     * Entry point to this test case.
     *
     * @param args parameters passed from command line
     */
    public static void main(String[] args) {
        new ScriptContextClassTest().doTests(args);
    }
}

