/*
  Rhino test framework

   Copyright (C) 2011, 2012  Red Hat

This file is part of IcedTea.

IcedTea is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2, or (at your option)
any later version.

IcedTea is distributed in the hope that it will be useful, but
WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
General Public License for more details.

You should have received a copy of the GNU General Public License
along with IcedTea; see the file COPYING.  If not, write to the
Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
02110-1301 USA.

Linking this library statically or dynamically with other modules is
making a combined work based on this library.  Thus, the terms and
conditions of the GNU General Public License cover the whole
combination.

As a special exception, the copyright holders of this library give you
permission to link this library with independent modules to produce an
executable, regardless of the license terms of these independent
modules, and to copy and distribute the resulting executable under
terms of your choice, provided that you also meet, for each linked
independent module, the terms and conditions of the license of that
module.  An independent module is a module which is not derived from
or based on this library.  If you modify this library, you may extend
this exception to your version of the library, but you are not
obligated to do so.  If you do not wish to do so, delete this
exception statement from your version.
*/

package org.RhinoTests;

import java.util.Arrays;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.HashMap;

import java.lang.reflect.Constructor;
import java.lang.reflect.Field;
import java.lang.reflect.Method;
import java.lang.reflect.Modifier;

import javax.script.ScriptEngine;
import javax.script.ScriptEngineManager;
import javax.script.ScriptEngine;



/**
 * Set of tests which check the API of ScriptEngine interface using
 * Java reflection API.
 *
 * @author Pavel Tisnovsky
 */
public class ScriptEngineClassTest extends BaseRhinoTest {

    /**
     * Object that represents the type of ScriptEngine.
     */
    Class<?> scriptEngineClass = null;

    @Override
    protected void setUp(String[] args) {
        // setup attribute used by tests
        this.scriptEngineClass = ScriptEngine.class;
    }

    @Override
    protected void tearDown() {
        // this block could be empty
        return;
    }

    /**
     * Test for method javax.script.ScriptEngine.getClass().isAssignableFrom()
     */
    protected void testIsAssignableFrom() {
        assertTrue(this.scriptEngineClass.isAssignableFrom(ScriptEngine.class),
                "Method ScriptEngine.getClass().isAssignableFrom() returns wrong value");
    }

    /**
     * Test for method javax.script.ScriptEngine.getClass().isInstance()
     */
    protected void testIsInstance() {
        assertTrue(this.scriptEngineClass.isInstance(new ScriptEngineManager().getEngineByName(Constants.EngineNames.ENGINE_NAME_JavaScript)),
                "Method ScriptEngine.getClass().isInstance() returns wrong value");
    }

    /**
     * Test for method javax.script.ScriptEngine.getClass().isInterface()
     */
    protected void testIsInterface() {
        assertTrue(this.scriptEngineClass.isInterface(),
                "Method ScriptEngine.getClass().isInterface() returns wrong value");
    }

    /**
     * Test for method javax.script.ScriptEngine.getClass().isLocalClass()
     */
    protected void testIsLocalClass() {
        assertFalse(this.scriptEngineClass.isLocalClass(),
                "Method ScriptEngine.getClass().isLocalClass() returns wrong value");
    }

    /**
     * Test for method javax.script.ScriptEngine.getClass().isMemberClass()
     */
    protected void testIsMemberClass() {
        assertFalse(this.scriptEngineClass.isMemberClass(),
                "Method ScriptEngine.getClass().isMemberClass() returns wrong value");
    }

    /**
     * Test for method javax.script.ScriptEngine.getClass().isPrimitive()
     */
    protected void testIsPrimitive() {
        assertFalse(this.scriptEngineClass.isPrimitive(),
                "Method ScriptEngine.getClass().isPrimitive() returns wrong value");
    }

    /**
     * Test for method javax.script.ScriptEngine.getClass().isSynthetic()
     */
    protected void testIsSynthetic() {
        assertFalse(this.scriptEngineClass.isSynthetic(),
                "Method ScriptEngine.getClass().isSynthetic() returns wrong value");
    }

    /**
     * Test for method javax.script.ScriptEngine.getClass().isAnnotation()
     */
    protected void testIsAnnotation() {
        assertFalse(this.scriptEngineClass.isAnnotation(),
                "Method ScriptEngine.getClass().isAnnotation() returns wrong value");
    }

    /**
     * Test for method javax.script.ScriptEngine.getClass().isAnnotationPresent()
     */
    protected void testIsAnnotationPresent() {
        assertFalse(this.scriptEngineClass.isAnnotationPresent(java.lang.annotation.Annotation.class),
                "Method ScriptEngine.getClass().isAnnotationPresent(java.lang.annotation.Annotation.class) returns wrong value");
        assertFalse(this.scriptEngineClass.isAnnotationPresent(java.lang.annotation.Documented.class),
                "Method ScriptEngine.getClass().isAnnotationPresent(java.lang.annotation.Documented.class) returns wrong value");
        assertFalse(this.scriptEngineClass.isAnnotationPresent(java.lang.annotation.Inherited.class),
                "Method ScriptEngine.getClass().isAnnotationPresent(java.lang.annotation.Inherited.class) returns wrong value");
        assertFalse(this.scriptEngineClass.isAnnotationPresent(java.lang.annotation.Retention.class),
                "Method ScriptEngine.getClass().isAnnotationPresent(java.lang.annotation.Retention.class) returns wrong value");
        assertFalse(this.scriptEngineClass.isAnnotationPresent(java.lang.annotation.Target.class),
                "Method ScriptEngine.getClass().isAnnotationPresent(java.lang.annotation.Target.class) returns wrong value");
        assertFalse(this.scriptEngineClass.isAnnotationPresent(java.lang.Deprecated.class),
                "Method ScriptEngine.getClass().isAnnotationPresent(java.lang.Deprecated.class) returns wrong value");
        assertFalse(this.scriptEngineClass.isAnnotationPresent(java.lang.Override.class),
                "Method ScriptEngine.getClass().isAnnotationPresent(java.lang.Override.class) returns wrong value");
        assertFalse(this.scriptEngineClass.isAnnotationPresent(java.lang.SuppressWarnings.class),
                "Method ScriptEngine.getClass().isAnnotationPresent(java.lang.SuppressWarnings.class) returns wrong value");
    }

    /**
     * Test for method javax.script.ScriptEngine.getClass().isAnonymousClass()
     */
    protected void testIsAnonymousClass() {
        assertFalse(this.scriptEngineClass.isAnonymousClass(),
                "Method ScriptEngine.getClass().isAnonymousClass() returns wrong value");
    }

    /**
     * Test for method javax.script.ScriptEngine.getClass().isArray()
     */
    protected void testIsArray() {
        assertFalse(this.scriptEngineClass.isArray(),
                "Method ScriptEngine.getClass().isArray() returns wrong value");
    }

    /**
     * Test for method javax.script.ScriptEngine.getClass().isEnum()
     */
    protected void testIsEnum() {
        assertFalse(this.scriptEngineClass.isEnum(),
                "Method ScriptEngine.getClass().isEnum() returns wrong value");
    }

    /**
     * Test for method javax.script.ScriptEngine.getClass().getInterfaces()
     */
    protected void testGetInterfaces() {
        List<Class<?>> interfaces = Arrays.asList(this.scriptEngineClass.getInterfaces());
        assertTrue(interfaces.isEmpty(),
                "list of implemented interfaces should be empty");
    }

    /**
     * Test for method javax.script.ScriptEngine.getClass().getModifiers()
     */
    protected void testGetModifiers() {
        int modifiers = this.scriptEngineClass.getModifiers();
        assertTrue(Modifier.isPublic(modifiers),
                "Method ScriptEngine.getClass().getModifiers() - isPublic modifier is set to a wrong value");
        assertFalse(Modifier.isPrivate(modifiers),
                "Method ScriptEngine.getClass().getModifiers() - isPrivate modifier is set to a wrong value");
        assertFalse(Modifier.isProtected(modifiers),
                "Method ScriptEngine.getClass().getModifiers() - isProtected modifier is set to a wrong value");
        assertTrue(Modifier.isAbstract(modifiers),
                "Method ScriptEngine.getClass().getModifiers() - isAbstract modifier is set to a wrong value");
        assertFalse(Modifier.isFinal(modifiers),
                "Method ScriptEngine.getClass().getModifiers() - isFinal modifier is set to a wrong value");
        assertTrue(Modifier.isInterface(modifiers),
                "Method ScriptEngine.getClass().getModifiers() - isInterface modifier is set to a wrong value");
        assertFalse(Modifier.isNative(modifiers),
                "Method ScriptEngine.getClass().getModifiers() - isNative modifier is set to a wrong value");
        assertFalse(Modifier.isStatic(modifiers),
                "Method ScriptEngine.getClass().getModifiers() - isStatic modifier is set to a wrong value");
        assertFalse(Modifier.isStrict(modifiers),
                "Method ScriptEngine.getClass().getModifiers() - isStrict modifier is set to a wrong value");
        assertFalse(Modifier.isSynchronized(modifiers),
                "Method ScriptEngine.getClass().getModifiers() - isSynchronized modifier is set to a wrong value");
        assertFalse(Modifier.isTransient(modifiers),
                "Method ScriptEngine.getClass().getModifiers() - isTransient modifier is set to a wrong value");
        assertFalse(Modifier.isVolatile(modifiers),
                "Method ScriptEngine.getClass().getModifiers() - isVolatile modifier is set to a wrong value");
    }

    /**
     * Test for method javax.script.ScriptEngine.getClass().getName()
     */
    protected void testGetName() {
        String name = this.scriptEngineClass.getName();
        assertEquals(name, "javax.script.ScriptEngine",
                "Method ScriptEngine.getClass().getName() returns wrong value " + name);
    }

    /**
     * Test for method javax.script.ScriptEngine.getClass().getPackage()
     */
    protected void testGetPackage() {
        Package p = this.scriptEngineClass.getPackage();
        String packageName = p.getName();
        assertEquals(packageName, "javax.script",
                "Method ScriptEngine.getClass().getPackage().getName() returns wrong value " + packageName);
    }

    /**
     * Test for method javax.script.ScriptEngine.getClass().getSimpleName()
     */
    protected void testGetSimpleName() {
        String simpleName = this.scriptEngineClass.getSimpleName();
        assertEquals(simpleName, "ScriptEngine",
                "Method ScriptEngine.getClass().getSimpleName() returns wrong value " + simpleName);
    }

    /**
     * Test for method javax.script.ScriptEngine.getClass().getSuperclass()
     */
    protected void testGetSuperclass() {
        Class<?> superClass = this.scriptEngineClass.getSuperclass();
        assertNull(superClass,
                "Method ScriptEngine.getClass().getSuperclass() does not return null");
    }

    /**
     * Test for method javax.script.ScriptEngine.getClass().getConstructors()
     */
    protected void testGetConstructors() {
        // map of constructors which should exists
        Map<String, String> testedConstructors = null;
        Map<String, String> testedConstructors_jdk6 = new HashMap<String, String>();
        Map<String, String> testedConstructors_jdk7 = new HashMap<String, String>();


        // get the right map containing constructor signatures
        testedConstructors = getJavaVersion() < 7 ? testedConstructors_jdk6 : testedConstructors_jdk7;

        // get all constructors for this class
        Constructor<?>[] constructors = this.scriptEngineClass.getConstructors();

        // basic check for a number of constructors
        assertEquals(constructors.length, 0, "no constructors should be set");

    }

    /**
     * Test for method javax.script.ScriptEngine.getClass().getDeclaredConstructors()
     */
    protected void testGetDeclaredConstructors() {
        // map of constructors which should exists
        Map<String, String> testedConstructors = null;
        Map<String, String> testedConstructors_jdk6 = new HashMap<String, String>();
        Map<String, String> testedConstructors_jdk7 = new HashMap<String, String>();


        // get the right map containing constructor signatures
        testedConstructors = getJavaVersion() < 7 ? testedConstructors_jdk6 : testedConstructors_jdk7;

        // get all declared constructors for this class
        Constructor<?>[] declaredConstructors = this.scriptEngineClass.getDeclaredConstructors();

        // basic check for a number of declared constructors
        assertEquals(declaredConstructors.length, 0, "no constructors should be set");

    }

    /**
     * Test for method javax.script.ScriptEngine.getClass().getFields()
     */
    protected void testGetFields() {
        // following fields should exists
        final String[] fieldsThatShouldExists = {
            "public static final java.lang.String javax.script.ScriptEngine.ARGV",
            "public static final java.lang.String javax.script.ScriptEngine.FILENAME",
            "public static final java.lang.String javax.script.ScriptEngine.ENGINE",
            "public static final java.lang.String javax.script.ScriptEngine.ENGINE_VERSION",
            "public static final java.lang.String javax.script.ScriptEngine.NAME",
            "public static final java.lang.String javax.script.ScriptEngine.LANGUAGE",
            "public static final java.lang.String javax.script.ScriptEngine.LANGUAGE_VERSION",
        };
        // get all fields
        Field[] fields = this.scriptEngineClass.getFields();
        // and transform the array into a list of field names
        List<String> fieldsAsString = new ArrayList<String>();
        for (Field field : fields) {
            fieldsAsString.add(field.toString());
        }
        // check if all required fields really exists
        for (String fieldThatShouldExists : fieldsThatShouldExists) {
            assertTrue(fieldsAsString.contains(fieldThatShouldExists),
                    "field " + fieldThatShouldExists + " not found");
        }
    }

    /**
     * Test for method javax.script.ScriptEngine.getClass().getDeclaredFields()
     */
    protected void testGetDeclaredFields() {
        // following fields should be declared
        final String[] fieldsThatShouldExists = {
            "public static final java.lang.String javax.script.ScriptEngine.ARGV",
            "public static final java.lang.String javax.script.ScriptEngine.FILENAME",
            "public static final java.lang.String javax.script.ScriptEngine.ENGINE",
            "public static final java.lang.String javax.script.ScriptEngine.ENGINE_VERSION",
            "public static final java.lang.String javax.script.ScriptEngine.NAME",
            "public static final java.lang.String javax.script.ScriptEngine.LANGUAGE",
            "public static final java.lang.String javax.script.ScriptEngine.LANGUAGE_VERSION",
        };
        // get all declared fields
        Field[] declaredFields = this.scriptEngineClass.getDeclaredFields();
        // and transform the array into a list of field names
        List<String> declaredFieldsAsString = new ArrayList<String>();
        for (Field field : declaredFields) {
            declaredFieldsAsString.add(field.toString());
        }
        // check if all required fields really exists
        for (String fieldThatShouldExists : fieldsThatShouldExists) {
            assertTrue(declaredFieldsAsString.contains(fieldThatShouldExists),
                    "field " + fieldThatShouldExists + " not found");
        }
    }

    /**
     * Test for method javax.script.ScriptEngine.getClass().getMethods()
     */
    protected void testGetMethods() {
        // following methods should be inherited
        final String[] methodsThatShouldExists_jdk6 = {
            "public abstract java.lang.Object javax.script.ScriptEngine.eval(java.io.Reader) throws javax.script.ScriptException",
            "public abstract java.lang.Object javax.script.ScriptEngine.eval(java.io.Reader,javax.script.Bindings) throws javax.script.ScriptException",
            "public abstract java.lang.Object javax.script.ScriptEngine.eval(java.io.Reader,javax.script.ScriptContext) throws javax.script.ScriptException",
            "public abstract java.lang.Object javax.script.ScriptEngine.eval(java.lang.String) throws javax.script.ScriptException",
            "public abstract java.lang.Object javax.script.ScriptEngine.eval(java.lang.String,javax.script.Bindings) throws javax.script.ScriptException",
            "public abstract java.lang.Object javax.script.ScriptEngine.eval(java.lang.String,javax.script.ScriptContext) throws javax.script.ScriptException",
            "public abstract java.lang.Object javax.script.ScriptEngine.get(java.lang.String)",
            "public abstract javax.script.Bindings javax.script.ScriptEngine.createBindings()",
            "public abstract javax.script.Bindings javax.script.ScriptEngine.getBindings(int)",
            "public abstract javax.script.ScriptContext javax.script.ScriptEngine.getContext()",
            "public abstract javax.script.ScriptEngineFactory javax.script.ScriptEngine.getFactory()",
            "public abstract void javax.script.ScriptEngine.put(java.lang.String,java.lang.Object)",
            "public abstract void javax.script.ScriptEngine.setBindings(javax.script.Bindings,int)",
            "public abstract void javax.script.ScriptEngine.setContext(javax.script.ScriptContext)",
        };

        final String[] methodsThatShouldExists_jdk7 = {
            "public abstract java.lang.Object javax.script.ScriptEngine.eval(java.io.Reader) throws javax.script.ScriptException",
            "public abstract java.lang.Object javax.script.ScriptEngine.eval(java.io.Reader,javax.script.Bindings) throws javax.script.ScriptException",
            "public abstract java.lang.Object javax.script.ScriptEngine.eval(java.io.Reader,javax.script.ScriptContext) throws javax.script.ScriptException",
            "public abstract java.lang.Object javax.script.ScriptEngine.eval(java.lang.String) throws javax.script.ScriptException",
            "public abstract java.lang.Object javax.script.ScriptEngine.eval(java.lang.String,javax.script.Bindings) throws javax.script.ScriptException",
            "public abstract java.lang.Object javax.script.ScriptEngine.eval(java.lang.String,javax.script.ScriptContext) throws javax.script.ScriptException",
            "public abstract java.lang.Object javax.script.ScriptEngine.get(java.lang.String)",
            "public abstract javax.script.Bindings javax.script.ScriptEngine.createBindings()",
            "public abstract javax.script.Bindings javax.script.ScriptEngine.getBindings(int)",
            "public abstract javax.script.ScriptContext javax.script.ScriptEngine.getContext()",
            "public abstract javax.script.ScriptEngineFactory javax.script.ScriptEngine.getFactory()",
            "public abstract void javax.script.ScriptEngine.put(java.lang.String,java.lang.Object)",
            "public abstract void javax.script.ScriptEngine.setBindings(javax.script.Bindings,int)",
            "public abstract void javax.script.ScriptEngine.setContext(javax.script.ScriptContext)",
        };

        // get all inherited methods
        Method[] methods = this.scriptEngineClass.getMethods();
        // and transform the array into a list of method names
        List<String> methodsAsString = new ArrayList<String>();
        for (Method method : methods) {
            methodsAsString.add(method.toString());
        }
        String[] methodsThatShouldExists = getJavaVersion() < 7 ? methodsThatShouldExists_jdk6 : methodsThatShouldExists_jdk7;
        // check if all required methods really exists
        for (String methodThatShouldExists : methodsThatShouldExists) {
            assertTrue(methodsAsString.contains(methodThatShouldExists),
                    "method " + methodThatShouldExists + " not found");
        }
    }

    /**
     * Test for method javax.script.ScriptEngine.getClass().getDeclaredMethods()
     */
    protected void testGetDeclaredMethods() {
        // following methods should be declared
        final String[] declaredMethodsThatShouldExists_jdk6 = {
            "public abstract java.lang.Object javax.script.ScriptEngine.eval(java.io.Reader) throws javax.script.ScriptException",
            "public abstract java.lang.Object javax.script.ScriptEngine.eval(java.io.Reader,javax.script.Bindings) throws javax.script.ScriptException",
            "public abstract java.lang.Object javax.script.ScriptEngine.eval(java.io.Reader,javax.script.ScriptContext) throws javax.script.ScriptException",
            "public abstract java.lang.Object javax.script.ScriptEngine.eval(java.lang.String) throws javax.script.ScriptException",
            "public abstract java.lang.Object javax.script.ScriptEngine.eval(java.lang.String,javax.script.Bindings) throws javax.script.ScriptException",
            "public abstract java.lang.Object javax.script.ScriptEngine.eval(java.lang.String,javax.script.ScriptContext) throws javax.script.ScriptException",
            "public abstract java.lang.Object javax.script.ScriptEngine.get(java.lang.String)",
            "public abstract javax.script.Bindings javax.script.ScriptEngine.createBindings()",
            "public abstract javax.script.Bindings javax.script.ScriptEngine.getBindings(int)",
            "public abstract javax.script.ScriptContext javax.script.ScriptEngine.getContext()",
            "public abstract javax.script.ScriptEngineFactory javax.script.ScriptEngine.getFactory()",
            "public abstract void javax.script.ScriptEngine.put(java.lang.String,java.lang.Object)",
            "public abstract void javax.script.ScriptEngine.setBindings(javax.script.Bindings,int)",
            "public abstract void javax.script.ScriptEngine.setContext(javax.script.ScriptContext)",
        };

        final String[] declaredMethodsThatShouldExists_jdk7 = {
            "public abstract java.lang.Object javax.script.ScriptEngine.eval(java.io.Reader) throws javax.script.ScriptException",
            "public abstract java.lang.Object javax.script.ScriptEngine.eval(java.io.Reader,javax.script.Bindings) throws javax.script.ScriptException",
            "public abstract java.lang.Object javax.script.ScriptEngine.eval(java.io.Reader,javax.script.ScriptContext) throws javax.script.ScriptException",
            "public abstract java.lang.Object javax.script.ScriptEngine.eval(java.lang.String) throws javax.script.ScriptException",
            "public abstract java.lang.Object javax.script.ScriptEngine.eval(java.lang.String,javax.script.Bindings) throws javax.script.ScriptException",
            "public abstract java.lang.Object javax.script.ScriptEngine.eval(java.lang.String,javax.script.ScriptContext) throws javax.script.ScriptException",
            "public abstract java.lang.Object javax.script.ScriptEngine.get(java.lang.String)",
            "public abstract javax.script.Bindings javax.script.ScriptEngine.createBindings()",
            "public abstract javax.script.Bindings javax.script.ScriptEngine.getBindings(int)",
            "public abstract javax.script.ScriptContext javax.script.ScriptEngine.getContext()",
            "public abstract javax.script.ScriptEngineFactory javax.script.ScriptEngine.getFactory()",
            "public abstract void javax.script.ScriptEngine.put(java.lang.String,java.lang.Object)",
            "public abstract void javax.script.ScriptEngine.setBindings(javax.script.Bindings,int)",
            "public abstract void javax.script.ScriptEngine.setContext(javax.script.ScriptContext)",
        };

        // get all declared methods
        Method[] declaredMethods = this.scriptEngineClass.getDeclaredMethods();
        // and transform the array into a list of method names
        List<String> methodsAsString = new ArrayList<String>();
        for (Method method : declaredMethods) {
            methodsAsString.add(method.toString());
        }
        String[] declaredMethodsThatShouldExists = getJavaVersion() < 7 ? declaredMethodsThatShouldExists_jdk6 : declaredMethodsThatShouldExists_jdk7;
        // check if all required methods really exists
        for (String methodThatShouldExists : declaredMethodsThatShouldExists) {
            assertTrue(methodsAsString.contains(methodThatShouldExists),
                    "declared method " + methodThatShouldExists + " not found");
        }
    }

    /**
     * Test for instanceof operator applied to a class javax.script.ScriptEngine
     */
    @SuppressWarnings("cast")
    protected void testInstanceOf() {
        // tested object
        Object o = new ScriptEngineManager().getEngineByName(Constants.EngineNames.ENGINE_NAME_JavaScript);

        // basic check of instanceof operator
        assertTrue(o instanceof ScriptEngine, "instanceof ScriptEngine is wrongly evaluated to false");

        // check operator instanceof against all superclasses
        assertTrue(o instanceof Object, "instanceof Object is wrongly evaluated to false");
    }

    /**
     * Entry point to this test case.
     *
     * @param args parameters passed from command line
     */
    public static void main(String[] args) {
        new ScriptEngineClassTest().doTests(args);
    }
}

