/*
  Rhino test framework

   Copyright (C) 2011, 2012  Red Hat

This file is part of IcedTea.

IcedTea is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2, or (at your option)
any later version.

IcedTea is distributed in the hope that it will be useful, but
WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
General Public License for more details.

You should have received a copy of the GNU General Public License
along with IcedTea; see the file COPYING.  If not, write to the
Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
02110-1301 USA.

Linking this library statically or dynamically with other modules is
making a combined work based on this library.  Thus, the terms and
conditions of the GNU General Public License cover the whole
combination.

As a special exception, the copyright holders of this library give you
permission to link this library with independent modules to produce an
executable, regardless of the license terms of these independent
modules, and to copy and distribute the resulting executable under
terms of your choice, provided that you also meet, for each linked
independent module, the terms and conditions of the license of that
module.  An independent module is a module which is not derived from
or based on this library.  If you modify this library, you may extend
this exception to your version of the library, but you are not
obligated to do so.  If you do not wish to do so, delete this
exception statement from your version.
*/

package org.RhinoTests;

import java.util.Arrays;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.HashMap;

import java.lang.reflect.Constructor;
import java.lang.reflect.Field;
import java.lang.reflect.Method;
import java.lang.reflect.Modifier;

import javax.script.ScriptEngineManager;




/**
 * Set of tests which check the API of ScriptEngineManager class using
 * Java reflection API.
 *
 * @author Pavel Tisnovsky
 */
public class ScriptEngineManagerClassTest extends BaseRhinoTest {

    /**
     * Object that represents the type of ScriptEngineManager.
     */
    Class<?> scriptEngineManagerClass = null;

    @Override
    protected void setUp(String[] args) {
        // setup attribute used by tests
        this.scriptEngineManagerClass = ScriptEngineManager.class;
    }

    @Override
    protected void tearDown() {
        // this block could be empty
        return;
    }

    /**
     * Test for method javax.script.ScriptEngineManager.getClass().isAssignableFrom()
     */
    protected void testIsAssignableFrom() {
        assertTrue(this.scriptEngineManagerClass.isAssignableFrom(ScriptEngineManager.class),
                "Method ScriptEngineManager.getClass().isAssignableFrom() returns wrong value");
    }

    /**
     * Test for method javax.script.ScriptEngineManager.getClass().isInstance()
     */
    protected void testIsInstance() {
        assertTrue(this.scriptEngineManagerClass.isInstance(new ScriptEngineManager()),
                "Method ScriptEngineManager.getClass().isInstance() returns wrong value");
    }

    /**
     * Test for method javax.script.ScriptEngineManager.getClass().isInterface()
     */
    protected void testIsInterface() {
        assertFalse(this.scriptEngineManagerClass.isInterface(),
                "Method ScriptEngineManager.getClass().isInterface() returns wrong value");
    }

    /**
     * Test for method javax.script.ScriptEngineManager.getClass().isLocalClass()
     */
    protected void testIsLocalClass() {
        assertFalse(this.scriptEngineManagerClass.isLocalClass(),
                "Method ScriptEngineManager.getClass().isLocalClass() returns wrong value");
    }

    /**
     * Test for method javax.script.ScriptEngineManager.getClass().isMemberClass()
     */
    protected void testIsMemberClass() {
        assertFalse(this.scriptEngineManagerClass.isMemberClass(),
                "Method ScriptEngineManager.getClass().isMemberClass() returns wrong value");
    }

    /**
     * Test for method javax.script.ScriptEngineManager.getClass().isPrimitive()
     */
    protected void testIsPrimitive() {
        assertFalse(this.scriptEngineManagerClass.isPrimitive(),
                "Method ScriptEngineManager.getClass().isPrimitive() returns wrong value");
    }

    /**
     * Test for method javax.script.ScriptEngineManager.getClass().isSynthetic()
     */
    protected void testIsSynthetic() {
        assertFalse(this.scriptEngineManagerClass.isSynthetic(),
                "Method ScriptEngineManager.getClass().isSynthetic() returns wrong value");
    }

    /**
     * Test for method javax.script.ScriptEngineManager.getClass().isAnnotation()
     */
    protected void testIsAnnotation() {
        assertFalse(this.scriptEngineManagerClass.isAnnotation(),
                "Method ScriptEngineManager.getClass().isAnnotation() returns wrong value");
    }

    /**
     * Test for method javax.script.ScriptEngineManager.getClass().isAnnotationPresent()
     */
    protected void testIsAnnotationPresent() {
        assertFalse(this.scriptEngineManagerClass.isAnnotationPresent(java.lang.annotation.Annotation.class),
                "Method ScriptEngineManager.getClass().isAnnotationPresent(java.lang.annotation.Annotation.class) returns wrong value");
        assertFalse(this.scriptEngineManagerClass.isAnnotationPresent(java.lang.annotation.Documented.class),
                "Method ScriptEngineManager.getClass().isAnnotationPresent(java.lang.annotation.Documented.class) returns wrong value");
        assertFalse(this.scriptEngineManagerClass.isAnnotationPresent(java.lang.annotation.Inherited.class),
                "Method ScriptEngineManager.getClass().isAnnotationPresent(java.lang.annotation.Inherited.class) returns wrong value");
        assertFalse(this.scriptEngineManagerClass.isAnnotationPresent(java.lang.annotation.Retention.class),
                "Method ScriptEngineManager.getClass().isAnnotationPresent(java.lang.annotation.Retention.class) returns wrong value");
        assertFalse(this.scriptEngineManagerClass.isAnnotationPresent(java.lang.annotation.Target.class),
                "Method ScriptEngineManager.getClass().isAnnotationPresent(java.lang.annotation.Target.class) returns wrong value");
        assertFalse(this.scriptEngineManagerClass.isAnnotationPresent(java.lang.Deprecated.class),
                "Method ScriptEngineManager.getClass().isAnnotationPresent(java.lang.Deprecated.class) returns wrong value");
        assertFalse(this.scriptEngineManagerClass.isAnnotationPresent(java.lang.Override.class),
                "Method ScriptEngineManager.getClass().isAnnotationPresent(java.lang.Override.class) returns wrong value");
        assertFalse(this.scriptEngineManagerClass.isAnnotationPresent(java.lang.SuppressWarnings.class),
                "Method ScriptEngineManager.getClass().isAnnotationPresent(java.lang.SuppressWarnings.class) returns wrong value");
    }

    /**
     * Test for method javax.script.ScriptEngineManager.getClass().isAnonymousClass()
     */
    protected void testIsAnonymousClass() {
        assertFalse(this.scriptEngineManagerClass.isAnonymousClass(),
                "Method ScriptEngineManager.getClass().isAnonymousClass() returns wrong value");
    }

    /**
     * Test for method javax.script.ScriptEngineManager.getClass().isArray()
     */
    protected void testIsArray() {
        assertFalse(this.scriptEngineManagerClass.isArray(),
                "Method ScriptEngineManager.getClass().isArray() returns wrong value");
    }

    /**
     * Test for method javax.script.ScriptEngineManager.getClass().isEnum()
     */
    protected void testIsEnum() {
        assertFalse(this.scriptEngineManagerClass.isEnum(),
                "Method ScriptEngineManager.getClass().isEnum() returns wrong value");
    }

    /**
     * Test for method javax.script.ScriptEngineManager.getClass().getInterfaces()
     */
    protected void testGetInterfaces() {
        List<Class<?>> interfaces = Arrays.asList(this.scriptEngineManagerClass.getInterfaces());
        assertTrue(interfaces.isEmpty(),
                "list of implemented interfaces should be empty");
    }

    /**
     * Test for method javax.script.ScriptEngineManager.getClass().getModifiers()
     */
    protected void testGetModifiers() {
        int modifiers = this.scriptEngineManagerClass.getModifiers();
        assertTrue(Modifier.isPublic(modifiers),
                "Method ScriptEngineManager.getClass().getModifiers() - isPublic modifier is set to a wrong value");
        assertFalse(Modifier.isPrivate(modifiers),
                "Method ScriptEngineManager.getClass().getModifiers() - isPrivate modifier is set to a wrong value");
        assertFalse(Modifier.isProtected(modifiers),
                "Method ScriptEngineManager.getClass().getModifiers() - isProtected modifier is set to a wrong value");
        assertFalse(Modifier.isAbstract(modifiers),
                "Method ScriptEngineManager.getClass().getModifiers() - isAbstract modifier is set to a wrong value");
        assertFalse(Modifier.isFinal(modifiers),
                "Method ScriptEngineManager.getClass().getModifiers() - isFinal modifier is set to a wrong value");
        assertFalse(Modifier.isInterface(modifiers),
                "Method ScriptEngineManager.getClass().getModifiers() - isInterface modifier is set to a wrong value");
        assertFalse(Modifier.isNative(modifiers),
                "Method ScriptEngineManager.getClass().getModifiers() - isNative modifier is set to a wrong value");
        assertFalse(Modifier.isStatic(modifiers),
                "Method ScriptEngineManager.getClass().getModifiers() - isStatic modifier is set to a wrong value");
        assertFalse(Modifier.isStrict(modifiers),
                "Method ScriptEngineManager.getClass().getModifiers() - isStrict modifier is set to a wrong value");
        assertFalse(Modifier.isSynchronized(modifiers),
                "Method ScriptEngineManager.getClass().getModifiers() - isSynchronized modifier is set to a wrong value");
        assertFalse(Modifier.isTransient(modifiers),
                "Method ScriptEngineManager.getClass().getModifiers() - isTransient modifier is set to a wrong value");
        assertFalse(Modifier.isVolatile(modifiers),
                "Method ScriptEngineManager.getClass().getModifiers() - isVolatile modifier is set to a wrong value");
    }

    /**
     * Test for method javax.script.ScriptEngineManager.getClass().getName()
     */
    protected void testGetName() {
        String name = this.scriptEngineManagerClass.getName();
        assertEquals(name, "javax.script.ScriptEngineManager",
                "Method ScriptEngineManager.getClass().getName() returns wrong value " + name);
    }

    /**
     * Test for method javax.script.ScriptEngineManager.getClass().getPackage()
     */
    protected void testGetPackage() {
        Package p = this.scriptEngineManagerClass.getPackage();
        String packageName = p.getName();
        assertEquals(packageName, "javax.script",
                "Method ScriptEngineManager.getClass().getPackage().getName() returns wrong value " + packageName);
    }

    /**
     * Test for method javax.script.ScriptEngineManager.getClass().getSimpleName()
     */
    protected void testGetSimpleName() {
        String simpleName = this.scriptEngineManagerClass.getSimpleName();
        assertEquals(simpleName, "ScriptEngineManager",
                "Method ScriptEngineManager.getClass().getSimpleName() returns wrong value " + simpleName);
    }

    /**
     * Test for method javax.script.ScriptEngineManager.getClass().getSuperclass()
     */
    protected void testGetSuperclass() {
        Class<?> superClass = this.scriptEngineManagerClass.getSuperclass();
        String superClassName = superClass.getName();
        assertEquals(superClassName, "java.lang.Object",
                "Method ScriptEngineManager.getClass().getSuperclass() returns wrong value " + superClassName);
    }

    /**
     * Test for method javax.script.ScriptEngineManager.getClass().getConstructors()
     */
    protected void testGetConstructors() {
        // map of constructors which should exists
        Map<String, String> testedConstructors = null;
        Map<String, String> testedConstructors_jdk6 = new HashMap<String, String>();
        Map<String, String> testedConstructors_jdk7 = new HashMap<String, String>();

        testedConstructors_jdk6.put("public javax.script.ScriptEngineManager()", "javax.script.ScriptEngineManager");
        testedConstructors_jdk6.put("public javax.script.ScriptEngineManager(java.lang.ClassLoader)", "javax.script.ScriptEngineManager");

        testedConstructors_jdk7.put("public javax.script.ScriptEngineManager()", "javax.script.ScriptEngineManager");
        testedConstructors_jdk7.put("public javax.script.ScriptEngineManager(java.lang.ClassLoader)", "javax.script.ScriptEngineManager");

        // get the right map containing constructor signatures
        testedConstructors = getJavaVersion() < 7 ? testedConstructors_jdk6 : testedConstructors_jdk7;

        // get all constructors for this class
        Constructor<?>[] constructors = this.scriptEngineManagerClass.getConstructors();

        // basic check for a number of constructors
        assertEquals(constructors.length, 2, "only 2 constructors should be set");

        // check if all constructors exists
        for (Constructor<?> constructor : constructors) {
            String constructorName = constructor.getName();
            String constructorString = constructor.toString();
            assertTrue(testedConstructors.containsKey(constructorString), "wrong constructor.toString() " + constructorName);
            assertEquals(testedConstructors.get(constructorString), constructorName, "wrong constructor name " + constructorName);
        }
    }

    /**
     * Test for method javax.script.ScriptEngineManager.getClass().getDeclaredConstructors()
     */
    protected void testGetDeclaredConstructors() {
        // map of constructors which should exists
        Map<String, String> testedConstructors = null;
        Map<String, String> testedConstructors_jdk6 = new HashMap<String, String>();
        Map<String, String> testedConstructors_jdk7 = new HashMap<String, String>();

        testedConstructors_jdk6.put("public javax.script.ScriptEngineManager()", "javax.script.ScriptEngineManager");
        testedConstructors_jdk6.put("public javax.script.ScriptEngineManager(java.lang.ClassLoader)", "javax.script.ScriptEngineManager");

        testedConstructors_jdk7.put("public javax.script.ScriptEngineManager()", "javax.script.ScriptEngineManager");
        testedConstructors_jdk7.put("public javax.script.ScriptEngineManager(java.lang.ClassLoader)", "javax.script.ScriptEngineManager");

        // get the right map containing constructor signatures
        testedConstructors = getJavaVersion() < 7 ? testedConstructors_jdk6 : testedConstructors_jdk7;

        // get all declared constructors for this class
        Constructor<?>[] declaredConstructors = this.scriptEngineManagerClass.getDeclaredConstructors();

        // basic check for a number of declared constructors
        assertEquals(declaredConstructors.length, 2, "only 2 constructors should be set");

        // check if all declared constructors exists
        for (Constructor<?> declaredConstructor : declaredConstructors) {
            String constructorName = declaredConstructor.getName();
            String constructorString = declaredConstructor.toString();
            assertTrue(testedConstructors.containsKey(constructorString), "wrong constructor.toString() " + constructorName);
            assertEquals(testedConstructors.get(constructorString), constructorName, "wrong constructor name " + constructorName);
        }
    }

    /**
     * Test for method javax.script.ScriptEngineManager.getClass().getFields()
     */
    protected void testGetFields() {
        // following fields should exists
        final String[] fieldsThatShouldExists = {
        };
        // get all fields
        Field[] fields = this.scriptEngineManagerClass.getFields();
        // and transform the array into a list of field names
        List<String> fieldsAsString = new ArrayList<String>();
        for (Field field : fields) {
            fieldsAsString.add(field.toString());
        }
        // check if all required fields really exists
        for (String fieldThatShouldExists : fieldsThatShouldExists) {
            assertTrue(fieldsAsString.contains(fieldThatShouldExists),
                    "field " + fieldThatShouldExists + " not found");
        }
    }

    /**
     * Test for method javax.script.ScriptEngineManager.getClass().getDeclaredFields()
     */
    protected void testGetDeclaredFields() {
        // following fields should be declared
        final String[] fieldsThatShouldExists = {
            "private static final boolean javax.script.ScriptEngineManager.DEBUG",
            "private java.util.HashSet javax.script.ScriptEngineManager.engineSpis",
            "private java.util.HashMap javax.script.ScriptEngineManager.nameAssociations",
            "private java.util.HashMap javax.script.ScriptEngineManager.extensionAssociations",
            "private java.util.HashMap javax.script.ScriptEngineManager.mimeTypeAssociations",
            "private javax.script.Bindings javax.script.ScriptEngineManager.globalScope",
        };
        // get all declared fields
        Field[] declaredFields = this.scriptEngineManagerClass.getDeclaredFields();
        // and transform the array into a list of field names
        List<String> declaredFieldsAsString = new ArrayList<String>();
        for (Field field : declaredFields) {
            declaredFieldsAsString.add(field.toString());
        }
        // check if all required fields really exists
        for (String fieldThatShouldExists : fieldsThatShouldExists) {
            assertTrue(declaredFieldsAsString.contains(fieldThatShouldExists),
                    "field " + fieldThatShouldExists + " not found");
        }
    }

    /**
     * Test for method javax.script.ScriptEngineManager.getClass().getMethods()
     */
    protected void testGetMethods() {
        // following methods should be inherited
        final String[] methodsThatShouldExists_jdk6 = {
            "public boolean java.lang.Object.equals(java.lang.Object)",
            "public final native java.lang.Class java.lang.Object.getClass()",
            "public final native void java.lang.Object.notify()",
            "public final native void java.lang.Object.notifyAll()",
            "public final native void java.lang.Object.wait(long) throws java.lang.InterruptedException",
            "public final void java.lang.Object.wait() throws java.lang.InterruptedException",
            "public final void java.lang.Object.wait(long,int) throws java.lang.InterruptedException",
            "public java.lang.Object javax.script.ScriptEngineManager.get(java.lang.String)",
            "public java.lang.String java.lang.Object.toString()",
            "public java.util.List javax.script.ScriptEngineManager.getEngineFactories()",
            "public javax.script.Bindings javax.script.ScriptEngineManager.getBindings()",
            "public javax.script.ScriptEngine javax.script.ScriptEngineManager.getEngineByExtension(java.lang.String)",
            "public javax.script.ScriptEngine javax.script.ScriptEngineManager.getEngineByMimeType(java.lang.String)",
            "public javax.script.ScriptEngine javax.script.ScriptEngineManager.getEngineByName(java.lang.String)",
            "public native int java.lang.Object.hashCode()",
            "public void javax.script.ScriptEngineManager.put(java.lang.String,java.lang.Object)",
            "public void javax.script.ScriptEngineManager.registerEngineExtension(java.lang.String,javax.script.ScriptEngineFactory)",
            "public void javax.script.ScriptEngineManager.registerEngineMimeType(java.lang.String,javax.script.ScriptEngineFactory)",
            "public void javax.script.ScriptEngineManager.registerEngineName(java.lang.String,javax.script.ScriptEngineFactory)",
            "public void javax.script.ScriptEngineManager.setBindings(javax.script.Bindings)",
        };

        final String[] methodsThatShouldExists_jdk7 = {
            "public boolean java.lang.Object.equals(java.lang.Object)",
            "public final native java.lang.Class java.lang.Object.getClass()",
            "public final native void java.lang.Object.notify()",
            "public final native void java.lang.Object.notifyAll()",
            "public final native void java.lang.Object.wait(long) throws java.lang.InterruptedException",
            "public final void java.lang.Object.wait() throws java.lang.InterruptedException",
            "public final void java.lang.Object.wait(long,int) throws java.lang.InterruptedException",
            "public java.lang.Object javax.script.ScriptEngineManager.get(java.lang.String)",
            "public java.lang.String java.lang.Object.toString()",
            "public java.util.List javax.script.ScriptEngineManager.getEngineFactories()",
            "public javax.script.Bindings javax.script.ScriptEngineManager.getBindings()",
            "public javax.script.ScriptEngine javax.script.ScriptEngineManager.getEngineByExtension(java.lang.String)",
            "public javax.script.ScriptEngine javax.script.ScriptEngineManager.getEngineByMimeType(java.lang.String)",
            "public javax.script.ScriptEngine javax.script.ScriptEngineManager.getEngineByName(java.lang.String)",
            "public native int java.lang.Object.hashCode()",
            "public void javax.script.ScriptEngineManager.put(java.lang.String,java.lang.Object)",
            "public void javax.script.ScriptEngineManager.registerEngineExtension(java.lang.String,javax.script.ScriptEngineFactory)",
            "public void javax.script.ScriptEngineManager.registerEngineMimeType(java.lang.String,javax.script.ScriptEngineFactory)",
            "public void javax.script.ScriptEngineManager.registerEngineName(java.lang.String,javax.script.ScriptEngineFactory)",
            "public void javax.script.ScriptEngineManager.setBindings(javax.script.Bindings)",
        };

        // get all inherited methods
        Method[] methods = this.scriptEngineManagerClass.getMethods();
        // and transform the array into a list of method names
        List<String> methodsAsString = new ArrayList<String>();
        for (Method method : methods) {
            methodsAsString.add(method.toString());
        }
        String[] methodsThatShouldExists = getJavaVersion() < 7 ? methodsThatShouldExists_jdk6 : methodsThatShouldExists_jdk7;
        // check if all required methods really exists
        for (String methodThatShouldExists : methodsThatShouldExists) {
            assertTrue(methodsAsString.contains(methodThatShouldExists),
                    "method " + methodThatShouldExists + " not found");
        }
    }

    /**
     * Test for method javax.script.ScriptEngineManager.getClass().getDeclaredMethods()
     */
    protected void testGetDeclaredMethods() {
        // following methods should be declared
        final String[] declaredMethodsThatShouldExists_jdk6 = {
            "private boolean javax.script.ScriptEngineManager.canCallerAccessLoader(java.lang.ClassLoader)",
            "private boolean javax.script.ScriptEngineManager.isAncestor(java.lang.ClassLoader,java.lang.ClassLoader)",
            "private java.lang.ClassLoader javax.script.ScriptEngineManager.getCallerClassLoader()",
            "private void javax.script.ScriptEngineManager.init(java.lang.ClassLoader)",
            "private void javax.script.ScriptEngineManager.initEngines(java.lang.ClassLoader)",
            "public java.lang.Object javax.script.ScriptEngineManager.get(java.lang.String)",
            "public java.util.List javax.script.ScriptEngineManager.getEngineFactories()",
            "public javax.script.Bindings javax.script.ScriptEngineManager.getBindings()",
            "public javax.script.ScriptEngine javax.script.ScriptEngineManager.getEngineByExtension(java.lang.String)",
            "public javax.script.ScriptEngine javax.script.ScriptEngineManager.getEngineByMimeType(java.lang.String)",
            "public javax.script.ScriptEngine javax.script.ScriptEngineManager.getEngineByName(java.lang.String)",
            "public void javax.script.ScriptEngineManager.put(java.lang.String,java.lang.Object)",
            "public void javax.script.ScriptEngineManager.registerEngineExtension(java.lang.String,javax.script.ScriptEngineFactory)",
            "public void javax.script.ScriptEngineManager.registerEngineMimeType(java.lang.String,javax.script.ScriptEngineFactory)",
            "public void javax.script.ScriptEngineManager.registerEngineName(java.lang.String,javax.script.ScriptEngineFactory)",
            "public void javax.script.ScriptEngineManager.setBindings(javax.script.Bindings)",
            "static void javax.script.ScriptEngineManager.access$000(javax.script.ScriptEngineManager,java.lang.ClassLoader)",
        };

        final String[] declaredMethodsThatShouldExists_jdk7 = {
            "private boolean javax.script.ScriptEngineManager.canCallerAccessLoader(java.lang.ClassLoader)",
            "private boolean javax.script.ScriptEngineManager.isAncestor(java.lang.ClassLoader,java.lang.ClassLoader)",
            "private java.lang.ClassLoader javax.script.ScriptEngineManager.getCallerClassLoader()",
            "private void javax.script.ScriptEngineManager.init(java.lang.ClassLoader)",
            "private void javax.script.ScriptEngineManager.initEngines(java.lang.ClassLoader)",
            "public java.lang.Object javax.script.ScriptEngineManager.get(java.lang.String)",
            "public java.util.List javax.script.ScriptEngineManager.getEngineFactories()",
            "public javax.script.Bindings javax.script.ScriptEngineManager.getBindings()",
            "public javax.script.ScriptEngine javax.script.ScriptEngineManager.getEngineByExtension(java.lang.String)",
            "public javax.script.ScriptEngine javax.script.ScriptEngineManager.getEngineByMimeType(java.lang.String)",
            "public javax.script.ScriptEngine javax.script.ScriptEngineManager.getEngineByName(java.lang.String)",
            "public void javax.script.ScriptEngineManager.put(java.lang.String,java.lang.Object)",
            "public void javax.script.ScriptEngineManager.registerEngineExtension(java.lang.String,javax.script.ScriptEngineFactory)",
            "public void javax.script.ScriptEngineManager.registerEngineMimeType(java.lang.String,javax.script.ScriptEngineFactory)",
            "public void javax.script.ScriptEngineManager.registerEngineName(java.lang.String,javax.script.ScriptEngineFactory)",
            "public void javax.script.ScriptEngineManager.setBindings(javax.script.Bindings)",
            "static void javax.script.ScriptEngineManager.access$000(javax.script.ScriptEngineManager,java.lang.ClassLoader)",
        };

        // get all declared methods
        Method[] declaredMethods = this.scriptEngineManagerClass.getDeclaredMethods();
        // and transform the array into a list of method names
        List<String> methodsAsString = new ArrayList<String>();
        for (Method method : declaredMethods) {
            methodsAsString.add(method.toString());
        }
        String[] declaredMethodsThatShouldExists = getJavaVersion() < 7 ? declaredMethodsThatShouldExists_jdk6 : declaredMethodsThatShouldExists_jdk7;
        // check if all required methods really exists
        for (String methodThatShouldExists : declaredMethodsThatShouldExists) {
            assertTrue(methodsAsString.contains(methodThatShouldExists),
                    "declared method " + methodThatShouldExists + " not found");
        }
    }

    /**
     * Test for instanceof operator applied to a class javax.script.ScriptEngineManager
     */
    @SuppressWarnings("cast")
    protected void testInstanceOf() {
        // tested object
        Object o = new ScriptEngineManager();

        // basic check of instanceof operator
        assertTrue(o instanceof ScriptEngineManager, "instanceof ScriptEngineManager is wrongly evaluated to false");

        // check operator instanceof against all superclasses
        assertTrue(o instanceof Object, "instanceof Object is wrongly evaluated to false");
    }

    /**
     * Entry point to this test case.
     *
     * @param args parameters passed from command line
     */
    public static void main(String[] args) {
        new ScriptEngineManagerClassTest().doTests(args);
    }
}

