/*
  Rhino test framework

   Copyright (C) 2011, 2012  Red Hat

This file is part of IcedTea.

IcedTea is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2, or (at your option)
any later version.

IcedTea is distributed in the hope that it will be useful, but
WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
General Public License for more details.

You should have received a copy of the GNU General Public License
along with IcedTea; see the file COPYING.  If not, write to the
Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
02110-1301 USA.

Linking this library statically or dynamically with other modules is
making a combined work based on this library.  Thus, the terms and
conditions of the GNU General Public License cover the whole
combination.

As a special exception, the copyright holders of this library give you
permission to link this library with independent modules to produce an
executable, regardless of the license terms of these independent
modules, and to copy and distribute the resulting executable under
terms of your choice, provided that you also meet, for each linked
independent module, the terms and conditions of the license of that
module.  An independent module is a module which is not derived from
or based on this library.  If you modify this library, you may extend
this exception to your version of the library, but you are not
obligated to do so.  If you do not wish to do so, delete this
exception statement from your version.
*/

package org.RhinoTests;

import java.util.Arrays;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.HashMap;

import java.lang.reflect.Constructor;
import java.lang.reflect.Field;
import java.lang.reflect.Method;
import java.lang.reflect.Modifier;

import javax.script.SimpleBindings;
import javax.script.Bindings;



/**
 * Set of tests which check the API of SimpleBindings class using
 * Java reflection API.
 *
 * @author Pavel Tisnovsky
 */
public class SimpleBindingsClassTest extends BaseRhinoTest {

    /**
     * Object that represents the type of SimpleBindings.
     */
    Class<?> simpleBindingsClass = null;

    @Override
    protected void setUp(String[] args) {
        // setup attribute used by tests
        this.simpleBindingsClass = SimpleBindings.class;
    }

    @Override
    protected void tearDown() {
        // this block could be empty
        return;
    }

    /**
     * Test for method javax.script.SimpleBindings.getClass().isAssignableFrom()
     */
    protected void testIsAssignableFrom() {
        assertTrue(this.simpleBindingsClass.isAssignableFrom(SimpleBindings.class),
                "Method SimpleBindings.getClass().isAssignableFrom() returns wrong value");
    }

    /**
     * Test for method javax.script.SimpleBindings.getClass().isInstance()
     */
    protected void testIsInstance() {
        assertTrue(this.simpleBindingsClass.isInstance(new SimpleBindings()),
                "Method SimpleBindings.getClass().isInstance() returns wrong value");
    }

    /**
     * Test for method javax.script.SimpleBindings.getClass().isInterface()
     */
    protected void testIsInterface() {
        assertFalse(this.simpleBindingsClass.isInterface(),
                "Method SimpleBindings.getClass().isInterface() returns wrong value");
    }

    /**
     * Test for method javax.script.SimpleBindings.getClass().isLocalClass()
     */
    protected void testIsLocalClass() {
        assertFalse(this.simpleBindingsClass.isLocalClass(),
                "Method SimpleBindings.getClass().isLocalClass() returns wrong value");
    }

    /**
     * Test for method javax.script.SimpleBindings.getClass().isMemberClass()
     */
    protected void testIsMemberClass() {
        assertFalse(this.simpleBindingsClass.isMemberClass(),
                "Method SimpleBindings.getClass().isMemberClass() returns wrong value");
    }

    /**
     * Test for method javax.script.SimpleBindings.getClass().isPrimitive()
     */
    protected void testIsPrimitive() {
        assertFalse(this.simpleBindingsClass.isPrimitive(),
                "Method SimpleBindings.getClass().isPrimitive() returns wrong value");
    }

    /**
     * Test for method javax.script.SimpleBindings.getClass().isSynthetic()
     */
    protected void testIsSynthetic() {
        assertFalse(this.simpleBindingsClass.isSynthetic(),
                "Method SimpleBindings.getClass().isSynthetic() returns wrong value");
    }

    /**
     * Test for method javax.script.SimpleBindings.getClass().isAnnotation()
     */
    protected void testIsAnnotation() {
        assertFalse(this.simpleBindingsClass.isAnnotation(),
                "Method SimpleBindings.getClass().isAnnotation() returns wrong value");
    }

    /**
     * Test for method javax.script.SimpleBindings.getClass().isAnnotationPresent()
     */
    protected void testIsAnnotationPresent() {
        assertFalse(this.simpleBindingsClass.isAnnotationPresent(java.lang.annotation.Annotation.class),
                "Method SimpleBindings.getClass().isAnnotationPresent(java.lang.annotation.Annotation.class) returns wrong value");
        assertFalse(this.simpleBindingsClass.isAnnotationPresent(java.lang.annotation.Documented.class),
                "Method SimpleBindings.getClass().isAnnotationPresent(java.lang.annotation.Documented.class) returns wrong value");
        assertFalse(this.simpleBindingsClass.isAnnotationPresent(java.lang.annotation.Inherited.class),
                "Method SimpleBindings.getClass().isAnnotationPresent(java.lang.annotation.Inherited.class) returns wrong value");
        assertFalse(this.simpleBindingsClass.isAnnotationPresent(java.lang.annotation.Retention.class),
                "Method SimpleBindings.getClass().isAnnotationPresent(java.lang.annotation.Retention.class) returns wrong value");
        assertFalse(this.simpleBindingsClass.isAnnotationPresent(java.lang.annotation.Target.class),
                "Method SimpleBindings.getClass().isAnnotationPresent(java.lang.annotation.Target.class) returns wrong value");
        assertFalse(this.simpleBindingsClass.isAnnotationPresent(java.lang.Deprecated.class),
                "Method SimpleBindings.getClass().isAnnotationPresent(java.lang.Deprecated.class) returns wrong value");
        assertFalse(this.simpleBindingsClass.isAnnotationPresent(java.lang.Override.class),
                "Method SimpleBindings.getClass().isAnnotationPresent(java.lang.Override.class) returns wrong value");
        assertFalse(this.simpleBindingsClass.isAnnotationPresent(java.lang.SuppressWarnings.class),
                "Method SimpleBindings.getClass().isAnnotationPresent(java.lang.SuppressWarnings.class) returns wrong value");
    }

    /**
     * Test for method javax.script.SimpleBindings.getClass().isAnonymousClass()
     */
    protected void testIsAnonymousClass() {
        assertFalse(this.simpleBindingsClass.isAnonymousClass(),
                "Method SimpleBindings.getClass().isAnonymousClass() returns wrong value");
    }

    /**
     * Test for method javax.script.SimpleBindings.getClass().isArray()
     */
    protected void testIsArray() {
        assertFalse(this.simpleBindingsClass.isArray(),
                "Method SimpleBindings.getClass().isArray() returns wrong value");
    }

    /**
     * Test for method javax.script.SimpleBindings.getClass().isEnum()
     */
    protected void testIsEnum() {
        assertFalse(this.simpleBindingsClass.isEnum(),
                "Method SimpleBindings.getClass().isEnum() returns wrong value");
    }

    /**
     * Test for method javax.script.SimpleBindings.getClass().getInterfaces()
     */
    protected void testGetInterfaces() {
        List<Class<?>> interfaces = Arrays.asList(this.simpleBindingsClass.getInterfaces());
        assertTrue(interfaces.contains(Bindings.class),
                "list of implemented interfaces does not contain Bindings");
    }

    /**
     * Test for method javax.script.SimpleBindings.getClass().getModifiers()
     */
    protected void testGetModifiers() {
        int modifiers = this.simpleBindingsClass.getModifiers();
        assertTrue(Modifier.isPublic(modifiers),
                "Method SimpleBindings.getClass().getModifiers() - isPublic modifier is set to a wrong value");
        assertFalse(Modifier.isPrivate(modifiers),
                "Method SimpleBindings.getClass().getModifiers() - isPrivate modifier is set to a wrong value");
        assertFalse(Modifier.isProtected(modifiers),
                "Method SimpleBindings.getClass().getModifiers() - isProtected modifier is set to a wrong value");
        assertFalse(Modifier.isAbstract(modifiers),
                "Method SimpleBindings.getClass().getModifiers() - isAbstract modifier is set to a wrong value");
        assertFalse(Modifier.isFinal(modifiers),
                "Method SimpleBindings.getClass().getModifiers() - isFinal modifier is set to a wrong value");
        assertFalse(Modifier.isInterface(modifiers),
                "Method SimpleBindings.getClass().getModifiers() - isInterface modifier is set to a wrong value");
        assertFalse(Modifier.isNative(modifiers),
                "Method SimpleBindings.getClass().getModifiers() - isNative modifier is set to a wrong value");
        assertFalse(Modifier.isStatic(modifiers),
                "Method SimpleBindings.getClass().getModifiers() - isStatic modifier is set to a wrong value");
        assertFalse(Modifier.isStrict(modifiers),
                "Method SimpleBindings.getClass().getModifiers() - isStrict modifier is set to a wrong value");
        assertFalse(Modifier.isSynchronized(modifiers),
                "Method SimpleBindings.getClass().getModifiers() - isSynchronized modifier is set to a wrong value");
        assertFalse(Modifier.isTransient(modifiers),
                "Method SimpleBindings.getClass().getModifiers() - isTransient modifier is set to a wrong value");
        assertFalse(Modifier.isVolatile(modifiers),
                "Method SimpleBindings.getClass().getModifiers() - isVolatile modifier is set to a wrong value");
    }

    /**
     * Test for method javax.script.SimpleBindings.getClass().getName()
     */
    protected void testGetName() {
        String name = this.simpleBindingsClass.getName();
        assertEquals(name, "javax.script.SimpleBindings",
                "Method SimpleBindings.getClass().getName() returns wrong value " + name);
    }

    /**
     * Test for method javax.script.SimpleBindings.getClass().getPackage()
     */
    protected void testGetPackage() {
        Package p = this.simpleBindingsClass.getPackage();
        String packageName = p.getName();
        assertEquals(packageName, "javax.script",
                "Method SimpleBindings.getClass().getPackage().getName() returns wrong value " + packageName);
    }

    /**
     * Test for method javax.script.SimpleBindings.getClass().getSimpleName()
     */
    protected void testGetSimpleName() {
        String simpleName = this.simpleBindingsClass.getSimpleName();
        assertEquals(simpleName, "SimpleBindings",
                "Method SimpleBindings.getClass().getSimpleName() returns wrong value " + simpleName);
    }

    /**
     * Test for method javax.script.SimpleBindings.getClass().getSuperclass()
     */
    protected void testGetSuperclass() {
        Class<?> superClass = this.simpleBindingsClass.getSuperclass();
        String superClassName = superClass.getName();
        assertEquals(superClassName, "java.lang.Object",
                "Method SimpleBindings.getClass().getSuperclass() returns wrong value " + superClassName);
    }

    /**
     * Test for method javax.script.SimpleBindings.getClass().getConstructors()
     */
    protected void testGetConstructors() {
        // map of constructors which should exists
        Map<String, String> testedConstructors = null;
        Map<String, String> testedConstructors_jdk6 = new HashMap<String, String>();
        Map<String, String> testedConstructors_jdk7 = new HashMap<String, String>();

        testedConstructors_jdk6.put("public javax.script.SimpleBindings(java.util.Map)", "javax.script.SimpleBindings");
        testedConstructors_jdk6.put("public javax.script.SimpleBindings()", "javax.script.SimpleBindings");

        testedConstructors_jdk7.put("public javax.script.SimpleBindings(java.util.Map)", "javax.script.SimpleBindings");
        testedConstructors_jdk7.put("public javax.script.SimpleBindings()", "javax.script.SimpleBindings");

        // get the right map containing constructor signatures
        testedConstructors = getJavaVersion() < 7 ? testedConstructors_jdk6 : testedConstructors_jdk7;

        // get all constructors for this class
        Constructor<?>[] constructors = this.simpleBindingsClass.getConstructors();

        // basic check for a number of constructors
        assertEquals(constructors.length, 2, "only 2 constructors should be set");

        // check if all constructors exists
        for (Constructor<?> constructor : constructors) {
            String constructorName = constructor.getName();
            String constructorString = constructor.toString();
            assertTrue(testedConstructors.containsKey(constructorString), "wrong constructor.toString() " + constructorName);
            assertEquals(testedConstructors.get(constructorString), constructorName, "wrong constructor name " + constructorName);
        }
    }

    /**
     * Test for method javax.script.SimpleBindings.getClass().getDeclaredConstructors()
     */
    protected void testGetDeclaredConstructors() {
        // map of constructors which should exists
        Map<String, String> testedConstructors = null;
        Map<String, String> testedConstructors_jdk6 = new HashMap<String, String>();
        Map<String, String> testedConstructors_jdk7 = new HashMap<String, String>();

        testedConstructors_jdk6.put("public javax.script.SimpleBindings(java.util.Map)", "javax.script.SimpleBindings");
        testedConstructors_jdk6.put("public javax.script.SimpleBindings()", "javax.script.SimpleBindings");

        testedConstructors_jdk7.put("public javax.script.SimpleBindings(java.util.Map)", "javax.script.SimpleBindings");
        testedConstructors_jdk7.put("public javax.script.SimpleBindings()", "javax.script.SimpleBindings");

        // get the right map containing constructor signatures
        testedConstructors = getJavaVersion() < 7 ? testedConstructors_jdk6 : testedConstructors_jdk7;

        // get all declared constructors for this class
        Constructor<?>[] declaredConstructors = this.simpleBindingsClass.getDeclaredConstructors();

        // basic check for a number of declared constructors
        assertEquals(declaredConstructors.length, 2, "only 2 constructors should be set");

        // check if all declared constructors exists
        for (Constructor<?> declaredConstructor : declaredConstructors) {
            String constructorName = declaredConstructor.getName();
            String constructorString = declaredConstructor.toString();
            assertTrue(testedConstructors.containsKey(constructorString), "wrong constructor.toString() " + constructorName);
            assertEquals(testedConstructors.get(constructorString), constructorName, "wrong constructor name " + constructorName);
        }
    }

    /**
     * Test for method javax.script.SimpleBindings.getClass().getFields()
     */
    protected void testGetFields() {
        // following fields should exists
        final String[] fieldsThatShouldExists = {
        };
        // get all fields
        Field[] fields = this.simpleBindingsClass.getFields();
        // and transform the array into a list of field names
        List<String> fieldsAsString = new ArrayList<String>();
        for (Field field : fields) {
            fieldsAsString.add(field.toString());
        }
        // check if all required fields really exists
        for (String fieldThatShouldExists : fieldsThatShouldExists) {
            assertTrue(fieldsAsString.contains(fieldThatShouldExists),
                    "field " + fieldThatShouldExists + " not found");
        }
    }

    /**
     * Test for method javax.script.SimpleBindings.getClass().getDeclaredFields()
     */
    protected void testGetDeclaredFields() {
        // following fields should be declared
        final String[] fieldsThatShouldExists = {
            "private java.util.Map javax.script.SimpleBindings.map",
        };
        // get all declared fields
        Field[] declaredFields = this.simpleBindingsClass.getDeclaredFields();
        // and transform the array into a list of field names
        List<String> declaredFieldsAsString = new ArrayList<String>();
        for (Field field : declaredFields) {
            declaredFieldsAsString.add(field.toString());
        }
        // check if all required fields really exists
        for (String fieldThatShouldExists : fieldsThatShouldExists) {
            assertTrue(declaredFieldsAsString.contains(fieldThatShouldExists),
                    "field " + fieldThatShouldExists + " not found");
        }
    }

    /**
     * Test for method javax.script.SimpleBindings.getClass().getMethods()
     */
    protected void testGetMethods() {
        // following methods should be inherited
        final String[] methodsThatShouldExists_jdk6 = {
            "public boolean java.lang.Object.equals(java.lang.Object)",
            "public boolean javax.script.SimpleBindings.containsKey(java.lang.Object)",
            "public boolean javax.script.SimpleBindings.containsValue(java.lang.Object)",
            "public boolean javax.script.SimpleBindings.isEmpty()",
            "public final native java.lang.Class java.lang.Object.getClass()",
            "public final native void java.lang.Object.notify()",
            "public final native void java.lang.Object.notifyAll()",
            "public final native void java.lang.Object.wait(long) throws java.lang.InterruptedException",
            "public final void java.lang.Object.wait() throws java.lang.InterruptedException",
            "public final void java.lang.Object.wait(long,int) throws java.lang.InterruptedException",
            "public int javax.script.SimpleBindings.size()",
            "public java.lang.Object javax.script.SimpleBindings.get(java.lang.Object)",
            "public java.lang.Object javax.script.SimpleBindings.put(java.lang.Object,java.lang.Object)",
            "public java.lang.Object javax.script.SimpleBindings.put(java.lang.String,java.lang.Object)",
            "public java.lang.Object javax.script.SimpleBindings.remove(java.lang.Object)",
            "public java.lang.String java.lang.Object.toString()",
            "public java.util.Collection javax.script.SimpleBindings.values()",
            "public java.util.Set javax.script.SimpleBindings.entrySet()",
            "public java.util.Set javax.script.SimpleBindings.keySet()",
            "public native int java.lang.Object.hashCode()",
            "public void javax.script.SimpleBindings.clear()",
            "public void javax.script.SimpleBindings.putAll(java.util.Map)",
        };

        final String[] methodsThatShouldExists_jdk7 = {
            "public boolean java.lang.Object.equals(java.lang.Object)",
            "public boolean javax.script.SimpleBindings.containsKey(java.lang.Object)",
            "public boolean javax.script.SimpleBindings.containsValue(java.lang.Object)",
            "public boolean javax.script.SimpleBindings.isEmpty()",
            "public final native java.lang.Class java.lang.Object.getClass()",
            "public final native void java.lang.Object.notify()",
            "public final native void java.lang.Object.notifyAll()",
            "public final native void java.lang.Object.wait(long) throws java.lang.InterruptedException",
            "public final void java.lang.Object.wait() throws java.lang.InterruptedException",
            "public final void java.lang.Object.wait(long,int) throws java.lang.InterruptedException",
            "public int javax.script.SimpleBindings.size()",
            "public java.lang.Object javax.script.SimpleBindings.get(java.lang.Object)",
            "public java.lang.Object javax.script.SimpleBindings.put(java.lang.Object,java.lang.Object)",
            "public java.lang.Object javax.script.SimpleBindings.put(java.lang.String,java.lang.Object)",
            "public java.lang.Object javax.script.SimpleBindings.remove(java.lang.Object)",
            "public java.lang.String java.lang.Object.toString()",
            "public java.util.Collection javax.script.SimpleBindings.values()",
            "public java.util.Set javax.script.SimpleBindings.entrySet()",
            "public java.util.Set javax.script.SimpleBindings.keySet()",
            "public native int java.lang.Object.hashCode()",
            "public void javax.script.SimpleBindings.clear()",
            "public void javax.script.SimpleBindings.putAll(java.util.Map)",
        };

        // get all inherited methods
        Method[] methods = this.simpleBindingsClass.getMethods();
        // and transform the array into a list of method names
        List<String> methodsAsString = new ArrayList<String>();
        for (Method method : methods) {
            methodsAsString.add(method.toString());
        }
        String[] methodsThatShouldExists = getJavaVersion() < 7 ? methodsThatShouldExists_jdk6 : methodsThatShouldExists_jdk7;
        // check if all required methods really exists
        for (String methodThatShouldExists : methodsThatShouldExists) {
            assertTrue(methodsAsString.contains(methodThatShouldExists),
                    "method " + methodThatShouldExists + " not found");
        }
    }

    /**
     * Test for method javax.script.SimpleBindings.getClass().getDeclaredMethods()
     */
    protected void testGetDeclaredMethods() {
        // following methods should be declared
        final String[] declaredMethodsThatShouldExists_jdk6 = {
            "private void javax.script.SimpleBindings.checkKey(java.lang.Object)",
            "public boolean javax.script.SimpleBindings.containsKey(java.lang.Object)",
            "public boolean javax.script.SimpleBindings.containsValue(java.lang.Object)",
            "public boolean javax.script.SimpleBindings.isEmpty()",
            "public int javax.script.SimpleBindings.size()",
            "public java.lang.Object javax.script.SimpleBindings.get(java.lang.Object)",
            "public java.lang.Object javax.script.SimpleBindings.put(java.lang.Object,java.lang.Object)",
            "public java.lang.Object javax.script.SimpleBindings.put(java.lang.String,java.lang.Object)",
            "public java.lang.Object javax.script.SimpleBindings.remove(java.lang.Object)",
            "public java.util.Collection javax.script.SimpleBindings.values()",
            "public java.util.Set javax.script.SimpleBindings.entrySet()",
            "public java.util.Set javax.script.SimpleBindings.keySet()",
            "public void javax.script.SimpleBindings.clear()",
            "public void javax.script.SimpleBindings.putAll(java.util.Map)",
        };

        final String[] declaredMethodsThatShouldExists_jdk7 = {
            "private void javax.script.SimpleBindings.checkKey(java.lang.Object)",
            "public boolean javax.script.SimpleBindings.containsKey(java.lang.Object)",
            "public boolean javax.script.SimpleBindings.containsValue(java.lang.Object)",
            "public boolean javax.script.SimpleBindings.isEmpty()",
            "public int javax.script.SimpleBindings.size()",
            "public java.lang.Object javax.script.SimpleBindings.get(java.lang.Object)",
            "public java.lang.Object javax.script.SimpleBindings.put(java.lang.Object,java.lang.Object)",
            "public java.lang.Object javax.script.SimpleBindings.put(java.lang.String,java.lang.Object)",
            "public java.lang.Object javax.script.SimpleBindings.remove(java.lang.Object)",
            "public java.util.Collection javax.script.SimpleBindings.values()",
            "public java.util.Set javax.script.SimpleBindings.entrySet()",
            "public java.util.Set javax.script.SimpleBindings.keySet()",
            "public void javax.script.SimpleBindings.clear()",
            "public void javax.script.SimpleBindings.putAll(java.util.Map)",
        };

        // get all declared methods
        Method[] declaredMethods = this.simpleBindingsClass.getDeclaredMethods();
        // and transform the array into a list of method names
        List<String> methodsAsString = new ArrayList<String>();
        for (Method method : declaredMethods) {
            methodsAsString.add(method.toString());
        }
        String[] declaredMethodsThatShouldExists = getJavaVersion() < 7 ? declaredMethodsThatShouldExists_jdk6 : declaredMethodsThatShouldExists_jdk7;
        // check if all required methods really exists
        for (String methodThatShouldExists : declaredMethodsThatShouldExists) {
            assertTrue(methodsAsString.contains(methodThatShouldExists),
                    "declared method " + methodThatShouldExists + " not found");
        }
    }

    /**
     * Test for instanceof operator applied to a class javax.script.SimpleBindings
     */
    @SuppressWarnings("cast")
    protected void testInstanceOf() {
        // tested object
        Object o = new SimpleBindings();

        // basic check of instanceof operator
        assertTrue(o instanceof SimpleBindings, "instanceof SimpleBindings is wrongly evaluated to false");

        // check operator instanceof against all superclasses
        assertTrue(o instanceof Object, "instanceof Object is wrongly evaluated to false");
    }

    /**
     * Entry point to this test case.
     *
     * @param args parameters passed from command line
     */
    public static void main(String[] args) {
        new SimpleBindingsClassTest().doTests(args);
    }
}

